"""Implementations for common converter types"""
import ctypes

class Converter( object ):
	"""Base class for Converter types
	
	Converter objects are callable objects used with the 
	OpenGL.wrapper.Wrapper class to simplify the wrapping
	of functions by collecting commonly used functionality
	into a reusable function.
	
	Each Converter has two (class) attributes:

		argNames -- list of argument names for initialisation
		indexLookups -- set of (indexname, argName,methodName) values 
			to lookup on wrapper.  These allow us to use argument-name
			references to refer to which arguments to use when 
			processing (prevents the need to revise pointers when 
			we change the API for a function).
	
	Converters can be any of the Wrapper API helper functions,
	so the callable interface can vary among Converter classes.
	"""
	argNames = ( )
	indexLookups = ( )
	def __init__( self, *args, **named ):
		"""Store arguments in attributes
		
		*args -- mapped to self.argNames in order to set attributes
		**named -- mapped to self.argNames by name to set attributes
		"""
		argNames = list(self.argNames)
		for a in self.argNames:
			if named.has_key( a ):
				setattr( self, a, named[a] )
				argNames.remove( a )
		for a,value in zip( argNames, args ):
			setattr( self, a, value )
	def finalise( self, wrapper ):
		"""Look up our indices (where appropriate)"""
		for indexname,argName,methodName in self.indexLookups:
			setattr( 
				self, indexname, 
				getattr(wrapper,methodName)(getattr( self, argName ))
			)

# Definitions of the abstract interfaces...
class PyConverter( Converter ):
	"""Converter sub-class for use in Wrapper.pyConverters
	
	This class just defines the interface for a pyConverter-style
	Converter object
	"""
	def __call__( self, incoming, function, arguments ):
		"""Convert incoming argument into compatable data-types
		
		incoming -- the Python argument for this parameter
		function -- the wrapper.Wrapper class we are supporting
		arguments -- the complete set of arguments passed to the 
			function
		
		
		"""
		raise NotImplemented( """%s class doesn't implement __call__"""%(
			self.__class__.__name__,
		))
	
class CConverter( Converter ):
	"""Converter sub-class for use in Wrapper.cConverters
	
	This class just defines the interface for a cConverter-style
	Converter object
	"""
	def __call__( self, pyArgs, index, baseOperation ):
		"""Calculate C-compatible Python object from Python arguments
		
		pyArgs -- set of Python argument objects converted by 
			pyConverters from the incoming arguments 
		index -- our index in baseOperation.cConverters
		baseOperation -- the Wrapper object which we are supporting
		"""
		raise NotImplemented( """%s class doesn't implement __call__"""%(
			self.__class__.__name__,
		))
class ReturnValues( Converter ):
	"""Converter sub-class for use as Wrapper.returnValues
	
	This class just defines the interface for a returnValues-style
	Converter object
	"""
	def __call__( self, result, baseOperation, pyArgs, cArgs ):
		"""Return a final value to the caller
		
		result -- the raw ctypes result value 
		baseOperation -- the Wrapper object which we are supporting
		pyArgs -- the set of Python arguments produced by pyConverters
		cArgs -- the set of C-compatible arguments produced by CConverter
		
		return the Python object for the final result
		"""
		raise NotImplemented( """%s class doesn't implement __call__"""%(
			self.__class__.__name__,
		))

# Now the concrete classes...
class CallFuncPyConverter( PyConverter ):
	"""PyConverter that takes a callable and calls it on incoming"""
	def __init__( self, function ):
		"""Store the function"""
		self.function = function 
	def __call__( self, incoming, function, argument ):
		"""Call our function on incoming"""
		return self.function( incoming )
class DefaultCConverter( CConverter ):
	"""NULL or Default CConverter, returns same-named Python argument
	
	Used primarily to allow for specifying a converter that explicitly
	says "use the default behaviour".  This is *not* a finalise-ing
	converter, it is passed in the index explicitly and just retrieves
	that index from pyArgs when called.  
	
	Raises informative errors if the index cannot be resolved in pyArgs
	"""
	def __init__( self, index ):
		"""Just store index for future access"""
		self.index = index 
	def __call__( self, pyArgs, index, wrapper ):
		"""Return pyArgs[self.index] or raise a ValueError"""
		try:
			return pyArgs[ self.index ]
		except IndexError, err:
			raise ValueError(
				"""%s expected parameter %r at index %r (%s), but pyArgs only length %s"""%(
				self.wrappedOperation.__name__,
				argName, 
				pyIndex, 
				", ".join( self.pyConverterNames ),
				len(pyArgs )
			))

class returnCArgument( ReturnValues ):
	"""ReturnValues returning the named cArgs value"""
	argNames = ('name',)
	indexLookups = [ ('index','name', 'cArgIndex' ), ]
	__slots__ = ( 'index', 'name' )
	def __call__( self, result, baseOperation, pyArgs, cArgs ):
		"""Retrieve cArgs[ self.index ]"""
		return cArgs[self.index]
	
class returnPyArgument( ReturnValues ):
	"""ReturnValues returning the named pyArgs value"""
	argNames = ('name',)
	indexLookups = [ ('index','name', 'pyArgIndex' ), ]
	__slots__ = ( 'index', 'name' )
	def __call__( self, result, baseOperation, pyArgs, cArgs ):
		"""Retrieve pyArgs[ self.index ]"""
		return pyArgs[self.index]


class getPyArgsName( CConverter ):
	"""CConverter returning named Python argument
	
	Intended for use in cConverters, the function returned 
	retrieves the named pyArg and returns it when called.
	"""
	argNames = ('name',)
	indexLookups = [ ('index','name', 'pyArgIndex' ), ]
	__slots__ = ( 'index', 'name')
	def __call__( self, pyArgs, index, baseOperation ):
		"""Return pyArgs[ self.index ]"""
		try:
			return pyArgs[ self.index ]
		except AttributeError, err:
			raise RuntimeError( """"Did not resolve parameter index for %r"""%(self.name))

class Output( CConverter ):
	"""CConverter generating static-size typed output arrays
	
	Produces an output array of given type (arrayType) and 
	size using self.lookup() to determine the size of the 
	array to be produced, where the lookup function is passed 
	as an initialisation argument.
	
	Provides also:
	
		oldStyleReturn( ... ) for use in the default case of
			PyOpenGL compatability mode, where result arrays of
			size (1,) are returned as scalar values.
	"""
	argNames = ('name','size','arrayType' )
	indexLookups = [ ('outIndex','name', 'cArgIndex' ), ]
	__slots__ = ('index','size','arrayType')
	def __call__( self, pyArgs, index, baseOperation ):
		"""Return pyArgs[ self.index ]"""
		return self.arrayType.zeros( self.getSize(pyArgs) )
	def getSize( self, pyArgs ):
		"""Retrieve the array size for this argument"""
		return self.size
	def oldStyleReturn( self, result, baseOperation, pyArgs, cArgs ):
		"""Retrieve cArgs[ self.index ]"""
		result = cArgs[ self.outIndex ]
		thisSize = self.getSize(pyArgs)
		if thisSize == (1,):
			try:
				return result[0]
			except TypeError, err:
				return result
		else:
			return result

class SizedOutput( Output ):
	"""Output generating dynamically-sized typed output arrays
	
	Takes an extra parameter "specifier", which is the name of
	a Python argument to be passed to the lookup function in order
	to determine the appropriate size for the output array.
	"""
	argNames = ('name','specifier','lookup','arrayType' )
	indexLookups = [ 
		('outIndex','name', 'cArgIndex' ),
		('index','specifier', 'pyArgIndex' ), 
	]
	__slots__ = ('index','specifier','lookup','arrayType')
	def getSize( self, pyArgs ):
		"""Retrieve the array size for this argument"""
		try:
			specifier = pyArgs[ self.index ]
		except AttributeError, err:
			raise RuntimeError( """"Did not resolve parameter index for %r"""%(self.name))
		else:
			try:
				return self.lookup( specifier )
			except KeyError, err:
				raise KeyError( """Unknown specifier %s"""%( specifier ))

class StringLengths( CConverter ):
	"""CConverter for processing array-of-pointers-to-strings data-type
	
	Converter is a CConverter for the array-of-lengths for a
	array-of-pointers-to-strings data-type used to pass a set
	of code fragments to the GLSL compiler.
	
	Provides also:
	
		stringArray -- PyConverter callable ensuring list-of-strings 
			format for the python argument
			
		stringArrayForC -- CResolver converting the array to 
			POINTER(c_char_p) format for passing to C
			
		totalCount -- CConverter callable giving count of string 
			pointers (that is, length of the pointer array)
	"""
	argNames = ('name',)
	indexLookups = [ ('index','name', 'pyArgIndex' ), ]
	__slots__ = ()
	def __call__( self, pyArgs, index, baseOperation ):
		"""Get array of length integers for string contents"""
		from OpenGL import arrays
		return arrays.GLintArray.asArray(
			[len(x) for x in pyArgs[self.index]]
		)
	def totalCount( self, pyArgs, index, baseOperation ):
		"""Get array of length integers for string contents"""
		return len(pyArgs[self.index])
	def stringArray( self, arg, baseOperation, args ):
		"""Create basic array-of-strings object from pyArg"""
		value = [str(x) for x in arg]
		return value
	def stringArrayForC( self, strings ):
		"""Create a ctypes pointer to char-pointer set"""
		from OpenGL import arrays
		result = (ctypes.c_char_p * len(strings))()
		for i,s in enumerate(strings):
			result[i] = arrays.GLcharARBArray.dataPointer(s)
		return result




'''OpenGL extension ARB.transpose_matrix

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.transpose_matrix to provide a more 
Python-friendly API
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions, wrapper
from OpenGL.GL import glget
import ctypes
from OpenGL.raw.GL.ARB.transpose_matrix import *
### END AUTOGENERATED SECTION
for typ,arrayType in (
	('d',arrays.GLdoubleArray),
	('f',arrays.GLfloatArray),
):
	for function in ('glLoadTransposeMatrix','glMultTransposeMatrix'):
		name = '%s%sARB'%(function,typ)
		globals()[name] = arrays.setInputArraySizeType(
			globals()[name],
			16,
			arrayType, 
			'm',
		)
		del function,name
	del typ,arrayType



'''OpenGL extension ATI.vertex_streams

Overview (from the spec)
	
	This extension adds the ability to handle sets of auxilliary
	vertex and normal coordinates. These sets of auxilliary
	coordinates are termed streams, and can be routed selectively
	into the blend stages provided by the vertex blending extension.
	This functionality enables software animation techniques such
	as keyframe vertex morphing.
	
	

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/ATI/vertex_streams.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ATI_vertex_streams'
GL_MAX_VERTEX_STREAMS_ATI = constant.Constant( 'GL_MAX_VERTEX_STREAMS_ATI', 0x876B )
GL_VERTEX_STREAM0_ATI = constant.Constant( 'GL_VERTEX_STREAM0_ATI', 0x876C )
GL_VERTEX_STREAM1_ATI = constant.Constant( 'GL_VERTEX_STREAM1_ATI', 0x876D )
GL_VERTEX_STREAM2_ATI = constant.Constant( 'GL_VERTEX_STREAM2_ATI', 0x876E )
GL_VERTEX_STREAM3_ATI = constant.Constant( 'GL_VERTEX_STREAM3_ATI', 0x876F )
GL_VERTEX_STREAM4_ATI = constant.Constant( 'GL_VERTEX_STREAM4_ATI', 0x8770 )
GL_VERTEX_STREAM5_ATI = constant.Constant( 'GL_VERTEX_STREAM5_ATI', 0x8771 )
GL_VERTEX_STREAM6_ATI = constant.Constant( 'GL_VERTEX_STREAM6_ATI', 0x8772 )
GL_VERTEX_STREAM7_ATI = constant.Constant( 'GL_VERTEX_STREAM7_ATI', 0x8773 )
GL_VERTEX_SOURCE_ATI = constant.Constant( 'GL_VERTEX_SOURCE_ATI', 0x8774 )
glVertexStream1sATI = platform.createExtensionFunction( 
	'glVertexStream1sATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLshort,),
	doc = 'glVertexStream1sATI( GLenum(stream), GLshort(x) ) -> None',
	argNames = ('stream', 'x',),
)

glVertexStream1svATI = platform.createExtensionFunction( 
	'glVertexStream1svATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLshortArray,),
	doc = 'glVertexStream1svATI( GLenum(stream), GLshortArray(coords) ) -> None',
	argNames = ('stream', 'coords',),
)

glVertexStream1iATI = platform.createExtensionFunction( 
	'glVertexStream1iATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLint,),
	doc = 'glVertexStream1iATI( GLenum(stream), GLint(x) ) -> None',
	argNames = ('stream', 'x',),
)

glVertexStream1ivATI = platform.createExtensionFunction( 
	'glVertexStream1ivATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLintArray,),
	doc = 'glVertexStream1ivATI( GLenum(stream), GLintArray(coords) ) -> None',
	argNames = ('stream', 'coords',),
)

glVertexStream1fATI = platform.createExtensionFunction( 
	'glVertexStream1fATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLfloat,),
	doc = 'glVertexStream1fATI( GLenum(stream), GLfloat(x) ) -> None',
	argNames = ('stream', 'x',),
)

glVertexStream1fvATI = platform.createExtensionFunction( 
	'glVertexStream1fvATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLfloatArray,),
	doc = 'glVertexStream1fvATI( GLenum(stream), GLfloatArray(coords) ) -> None',
	argNames = ('stream', 'coords',),
)

glVertexStream1dATI = platform.createExtensionFunction( 
	'glVertexStream1dATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLdouble,),
	doc = 'glVertexStream1dATI( GLenum(stream), GLdouble(x) ) -> None',
	argNames = ('stream', 'x',),
)

glVertexStream1dvATI = platform.createExtensionFunction( 
	'glVertexStream1dvATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLdoubleArray,),
	doc = 'glVertexStream1dvATI( GLenum(stream), GLdoubleArray(coords) ) -> None',
	argNames = ('stream', 'coords',),
)

glVertexStream2sATI = platform.createExtensionFunction( 
	'glVertexStream2sATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLshort, constants.GLshort,),
	doc = 'glVertexStream2sATI( GLenum(stream), GLshort(x), GLshort(y) ) -> None',
	argNames = ('stream', 'x', 'y',),
)

glVertexStream2svATI = platform.createExtensionFunction( 
	'glVertexStream2svATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLshortArray,),
	doc = 'glVertexStream2svATI( GLenum(stream), GLshortArray(coords) ) -> None',
	argNames = ('stream', 'coords',),
)

glVertexStream2iATI = platform.createExtensionFunction( 
	'glVertexStream2iATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLint, constants.GLint,),
	doc = 'glVertexStream2iATI( GLenum(stream), GLint(x), GLint(y) ) -> None',
	argNames = ('stream', 'x', 'y',),
)

glVertexStream2ivATI = platform.createExtensionFunction( 
	'glVertexStream2ivATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLintArray,),
	doc = 'glVertexStream2ivATI( GLenum(stream), GLintArray(coords) ) -> None',
	argNames = ('stream', 'coords',),
)

glVertexStream2fATI = platform.createExtensionFunction( 
	'glVertexStream2fATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLfloat, constants.GLfloat,),
	doc = 'glVertexStream2fATI( GLenum(stream), GLfloat(x), GLfloat(y) ) -> None',
	argNames = ('stream', 'x', 'y',),
)

glVertexStream2fvATI = platform.createExtensionFunction( 
	'glVertexStream2fvATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLfloatArray,),
	doc = 'glVertexStream2fvATI( GLenum(stream), GLfloatArray(coords) ) -> None',
	argNames = ('stream', 'coords',),
)

glVertexStream2dATI = platform.createExtensionFunction( 
	'glVertexStream2dATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLdouble, constants.GLdouble,),
	doc = 'glVertexStream2dATI( GLenum(stream), GLdouble(x), GLdouble(y) ) -> None',
	argNames = ('stream', 'x', 'y',),
)

glVertexStream2dvATI = platform.createExtensionFunction( 
	'glVertexStream2dvATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLdoubleArray,),
	doc = 'glVertexStream2dvATI( GLenum(stream), GLdoubleArray(coords) ) -> None',
	argNames = ('stream', 'coords',),
)

glVertexStream3sATI = platform.createExtensionFunction( 
	'glVertexStream3sATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLshort, constants.GLshort, constants.GLshort,),
	doc = 'glVertexStream3sATI( GLenum(stream), GLshort(x), GLshort(y), GLshort(z) ) -> None',
	argNames = ('stream', 'x', 'y', 'z',),
)

glVertexStream3svATI = platform.createExtensionFunction( 
	'glVertexStream3svATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLshortArray,),
	doc = 'glVertexStream3svATI( GLenum(stream), GLshortArray(coords) ) -> None',
	argNames = ('stream', 'coords',),
)

glVertexStream3iATI = platform.createExtensionFunction( 
	'glVertexStream3iATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLint, constants.GLint, constants.GLint,),
	doc = 'glVertexStream3iATI( GLenum(stream), GLint(x), GLint(y), GLint(z) ) -> None',
	argNames = ('stream', 'x', 'y', 'z',),
)

glVertexStream3ivATI = platform.createExtensionFunction( 
	'glVertexStream3ivATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLintArray,),
	doc = 'glVertexStream3ivATI( GLenum(stream), GLintArray(coords) ) -> None',
	argNames = ('stream', 'coords',),
)

glVertexStream3fATI = platform.createExtensionFunction( 
	'glVertexStream3fATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLfloat, constants.GLfloat, constants.GLfloat,),
	doc = 'glVertexStream3fATI( GLenum(stream), GLfloat(x), GLfloat(y), GLfloat(z) ) -> None',
	argNames = ('stream', 'x', 'y', 'z',),
)

glVertexStream3fvATI = platform.createExtensionFunction( 
	'glVertexStream3fvATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLfloatArray,),
	doc = 'glVertexStream3fvATI( GLenum(stream), GLfloatArray(coords) ) -> None',
	argNames = ('stream', 'coords',),
)

glVertexStream3dATI = platform.createExtensionFunction( 
	'glVertexStream3dATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLdouble, constants.GLdouble, constants.GLdouble,),
	doc = 'glVertexStream3dATI( GLenum(stream), GLdouble(x), GLdouble(y), GLdouble(z) ) -> None',
	argNames = ('stream', 'x', 'y', 'z',),
)

glVertexStream3dvATI = platform.createExtensionFunction( 
	'glVertexStream3dvATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLdoubleArray,),
	doc = 'glVertexStream3dvATI( GLenum(stream), GLdoubleArray(coords) ) -> None',
	argNames = ('stream', 'coords',),
)

glVertexStream4sATI = platform.createExtensionFunction( 
	'glVertexStream4sATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLshort, constants.GLshort, constants.GLshort, constants.GLshort,),
	doc = 'glVertexStream4sATI( GLenum(stream), GLshort(x), GLshort(y), GLshort(z), GLshort(w) ) -> None',
	argNames = ('stream', 'x', 'y', 'z', 'w',),
)

glVertexStream4svATI = platform.createExtensionFunction( 
	'glVertexStream4svATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLshortArray,),
	doc = 'glVertexStream4svATI( GLenum(stream), GLshortArray(coords) ) -> None',
	argNames = ('stream', 'coords',),
)

glVertexStream4iATI = platform.createExtensionFunction( 
	'glVertexStream4iATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLint, constants.GLint, constants.GLint, constants.GLint,),
	doc = 'glVertexStream4iATI( GLenum(stream), GLint(x), GLint(y), GLint(z), GLint(w) ) -> None',
	argNames = ('stream', 'x', 'y', 'z', 'w',),
)

glVertexStream4ivATI = platform.createExtensionFunction( 
	'glVertexStream4ivATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLintArray,),
	doc = 'glVertexStream4ivATI( GLenum(stream), GLintArray(coords) ) -> None',
	argNames = ('stream', 'coords',),
)

glVertexStream4fATI = platform.createExtensionFunction( 
	'glVertexStream4fATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLfloat, constants.GLfloat, constants.GLfloat, constants.GLfloat,),
	doc = 'glVertexStream4fATI( GLenum(stream), GLfloat(x), GLfloat(y), GLfloat(z), GLfloat(w) ) -> None',
	argNames = ('stream', 'x', 'y', 'z', 'w',),
)

glVertexStream4fvATI = platform.createExtensionFunction( 
	'glVertexStream4fvATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLfloatArray,),
	doc = 'glVertexStream4fvATI( GLenum(stream), GLfloatArray(coords) ) -> None',
	argNames = ('stream', 'coords',),
)

glVertexStream4dATI = platform.createExtensionFunction( 
	'glVertexStream4dATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLdouble, constants.GLdouble, constants.GLdouble, constants.GLdouble,),
	doc = 'glVertexStream4dATI( GLenum(stream), GLdouble(x), GLdouble(y), GLdouble(z), GLdouble(w) ) -> None',
	argNames = ('stream', 'x', 'y', 'z', 'w',),
)

glVertexStream4dvATI = platform.createExtensionFunction( 
	'glVertexStream4dvATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLdoubleArray,),
	doc = 'glVertexStream4dvATI( GLenum(stream), GLdoubleArray(coords) ) -> None',
	argNames = ('stream', 'coords',),
)

glNormalStream3bATI = platform.createExtensionFunction( 
	'glNormalStream3bATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLbyte, constants.GLbyte, constants.GLbyte,),
	doc = 'glNormalStream3bATI( GLenum(stream), GLbyte(nx), GLbyte(ny), GLbyte(nz) ) -> None',
	argNames = ('stream', 'nx', 'ny', 'nz',),
)

glNormalStream3bvATI = platform.createExtensionFunction( 
	'glNormalStream3bvATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLbyteArray,),
	doc = 'glNormalStream3bvATI( GLenum(stream), GLbyteArray(coords) ) -> None',
	argNames = ('stream', 'coords',),
)

glNormalStream3sATI = platform.createExtensionFunction( 
	'glNormalStream3sATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLshort, constants.GLshort, constants.GLshort,),
	doc = 'glNormalStream3sATI( GLenum(stream), GLshort(nx), GLshort(ny), GLshort(nz) ) -> None',
	argNames = ('stream', 'nx', 'ny', 'nz',),
)

glNormalStream3svATI = platform.createExtensionFunction( 
	'glNormalStream3svATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLshortArray,),
	doc = 'glNormalStream3svATI( GLenum(stream), GLshortArray(coords) ) -> None',
	argNames = ('stream', 'coords',),
)

glNormalStream3iATI = platform.createExtensionFunction( 
	'glNormalStream3iATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLint, constants.GLint, constants.GLint,),
	doc = 'glNormalStream3iATI( GLenum(stream), GLint(nx), GLint(ny), GLint(nz) ) -> None',
	argNames = ('stream', 'nx', 'ny', 'nz',),
)

glNormalStream3ivATI = platform.createExtensionFunction( 
	'glNormalStream3ivATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLintArray,),
	doc = 'glNormalStream3ivATI( GLenum(stream), GLintArray(coords) ) -> None',
	argNames = ('stream', 'coords',),
)

glNormalStream3fATI = platform.createExtensionFunction( 
	'glNormalStream3fATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLfloat, constants.GLfloat, constants.GLfloat,),
	doc = 'glNormalStream3fATI( GLenum(stream), GLfloat(nx), GLfloat(ny), GLfloat(nz) ) -> None',
	argNames = ('stream', 'nx', 'ny', 'nz',),
)

glNormalStream3fvATI = platform.createExtensionFunction( 
	'glNormalStream3fvATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLfloatArray,),
	doc = 'glNormalStream3fvATI( GLenum(stream), GLfloatArray(coords) ) -> None',
	argNames = ('stream', 'coords',),
)

glNormalStream3dATI = platform.createExtensionFunction( 
	'glNormalStream3dATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLdouble, constants.GLdouble, constants.GLdouble,),
	doc = 'glNormalStream3dATI( GLenum(stream), GLdouble(nx), GLdouble(ny), GLdouble(nz) ) -> None',
	argNames = ('stream', 'nx', 'ny', 'nz',),
)

glNormalStream3dvATI = platform.createExtensionFunction( 
	'glNormalStream3dvATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLdoubleArray,),
	doc = 'glNormalStream3dvATI( GLenum(stream), GLdoubleArray(coords) ) -> None',
	argNames = ('stream', 'coords',),
)

glClientActiveVertexStreamATI = platform.createExtensionFunction( 
	'glClientActiveVertexStreamATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum,),
	doc = 'glClientActiveVertexStreamATI( GLenum(stream) ) -> None',
	argNames = ('stream',),
)

glVertexBlendEnviATI = platform.createExtensionFunction( 
	'glVertexBlendEnviATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLint,),
	doc = 'glVertexBlendEnviATI( GLenum(pname), GLint(param) ) -> None',
	argNames = ('pname', 'param',),
)

glVertexBlendEnvfATI = platform.createExtensionFunction( 
	'glVertexBlendEnvfATI', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLfloat,),
	doc = 'glVertexBlendEnvfATI( GLenum(pname), GLfloat(param) ) -> None',
	argNames = ('pname', 'param',),
)


def glInitVertexStreamsATI():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension INGR.interlace_read

Overview (from the spec)
	
	This extension provides a way to skip rows of pixels when reading
	or copying pixel rectangles.  This extension is complementary to
	the EXT_interlace extension except that it has no affect on getting
	texture images.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/INGR/interlace_read.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_INGR_interlace_read'
GL_INTERLACE_READ_INGR = constant.Constant( 'GL_INTERLACE_READ_INGR', 0x8568 )
glget.addGLGetConstant( GL_INTERLACE_READ_INGR, (1,) )


def glInitInterlaceReadINGR():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension ARB.multitexture

This module customises the behaviour of the 
OpenGL.raw.GL.ARB.multitexture to provide a more 
Python-friendly API
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions, wrapper
from OpenGL.GL import glget
import ctypes
from OpenGL.raw.GL.ARB.multitexture import *
### END AUTOGENERATED SECTION
for typ,arrayType in (
	('d',arrays.GLdoubleArray),
	('f',arrays.GLfloatArray),
	('i',arrays.GLintArray),
	('s',arrays.GLshortArray),
):
	for size in (1,2,3,4):
		name = 'glMultiTexCoord%(size)s%(typ)svARB'%globals()
		globals()[name] = arrays.setInputArraySizeType(
			globals()[name],
			size,
			arrayType, 
			'v',
		)
		del size,name
	del typ,arrayType



"""Array-size annotations for OpenGL.raw.GLX

Automatically generated by the generateraw script, do not edit!
"""
from OpenGL.raw import GLX as raw

from ctypes import *
from OpenGL import platform, arrays
from OpenGL.constant import Constant
from OpenGL import constants as GLconstants
GLvoid = GLconstants.GLvoid

STRING = c_char_p
WSTRING = c_wchar_p


glMultiTexCoord1dvARB = arrays.setInputArraySizeType(
	raw.glMultiTexCoord1dvARB,
	1,
	arrays.GLdoubleArray,
	'v',
)

glMultiTexCoord1fvARB = arrays.setInputArraySizeType(
	raw.glMultiTexCoord1fvARB,
	1,
	arrays.GLfloatArray,
	'v',
)

glMultiTexCoord1ivARB = arrays.setInputArraySizeType(
	raw.glMultiTexCoord1ivARB,
	1,
	arrays.GLintArray,
	'v',
)

glMultiTexCoord1svARB = arrays.setInputArraySizeType(
	raw.glMultiTexCoord1svARB,
	1,
	arrays.GLshortArray,
	'v',
)

glMultiTexCoord2dvARB = arrays.setInputArraySizeType(
	raw.glMultiTexCoord2dvARB,
	2,
	arrays.GLdoubleArray,
	'v',
)

glMultiTexCoord2fvARB = arrays.setInputArraySizeType(
	raw.glMultiTexCoord2fvARB,
	2,
	arrays.GLfloatArray,
	'v',
)

glMultiTexCoord2ivARB = arrays.setInputArraySizeType(
	raw.glMultiTexCoord2ivARB,
	2,
	arrays.GLintArray,
	'v',
)

glMultiTexCoord2svARB = arrays.setInputArraySizeType(
	raw.glMultiTexCoord2svARB,
	2,
	arrays.GLshortArray,
	'v',
)

glMultiTexCoord3dvARB = arrays.setInputArraySizeType(
	raw.glMultiTexCoord3dvARB,
	3,
	arrays.GLdoubleArray,
	'v',
)

glMultiTexCoord3fvARB = arrays.setInputArraySizeType(
	raw.glMultiTexCoord3fvARB,
	3,
	arrays.GLfloatArray,
	'v',
)

glMultiTexCoord3ivARB = arrays.setInputArraySizeType(
	raw.glMultiTexCoord3ivARB,
	3,
	arrays.GLintArray,
	'v',
)

glMultiTexCoord3svARB = arrays.setInputArraySizeType(
	raw.glMultiTexCoord3svARB,
	3,
	arrays.GLshortArray,
	'v',
)

glMultiTexCoord4dvARB = arrays.setInputArraySizeType(
	raw.glMultiTexCoord4dvARB,
	4,
	arrays.GLdoubleArray,
	'v',
)

glMultiTexCoord4fvARB = arrays.setInputArraySizeType(
	raw.glMultiTexCoord4fvARB,
	4,
	arrays.GLfloatArray,
	'v',
)

glMultiTexCoord4ivARB = arrays.setInputArraySizeType(
	raw.glMultiTexCoord4ivARB,
	4,
	arrays.GLintArray,
	'v',
)

glMultiTexCoord4svARB = arrays.setInputArraySizeType(
	raw.glMultiTexCoord4svARB,
	4,
	arrays.GLshortArray,
	'v',
)

glXGetProcAddress = arrays.setInputArraySizeType(
	raw.glXGetProcAddress,
	None, # XXX Could not determine size of argument procname for glXGetProcAddress arrays.GLubyteArray
	arrays.GLubyteArray, 
	'procname',
)

__all__ = [
	'glMultiTexCoord1dvARB',
	'glMultiTexCoord1fvARB',
	'glMultiTexCoord1ivARB',
	'glMultiTexCoord1svARB',
	'glMultiTexCoord2dvARB',
	'glMultiTexCoord2fvARB',
	'glMultiTexCoord2ivARB',
	'glMultiTexCoord2svARB',
	'glMultiTexCoord3dvARB',
	'glMultiTexCoord3fvARB',
	'glMultiTexCoord3ivARB',
	'glMultiTexCoord3svARB',
	'glMultiTexCoord4dvARB',
	'glMultiTexCoord4fvARB',
	'glMultiTexCoord4ivARB',
	'glMultiTexCoord4svARB',
	'glXGetProcAddress'
]




'''OpenGL extension NV.half_float

Overview (from the spec)
	
	This extension introduces a new storage format and data type for
	half-precision (16-bit) floating-point quantities.  The floating-point
	format is very similar to the IEEE single-precision floating-point
	standard, except that it has only 5 exponent bits and 10 mantissa bits.
	Half-precision floats are smaller than full precision floats and provide a
	larger dynamic range than similarly-sized normalized scalar data types.
	
	This extension allows applications to use half-precision floating point
	data when specifying vertices or pixel data.  It adds new commands to
	specify vertex attributes using the new data type, and extends the
	existing vertex array and image specification commands to accept the new
	data type.
	
	This storage format is also used to represent 16-bit components in the
	floating-point frame buffers, as defined in the NV_float_buffer extension.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/NV/half_float.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_NV_half_float'
GL_HALF_FLOAT_NV = constant.Constant( 'GL_HALF_FLOAT_NV', 0x140B )
glVertex2hNV = platform.createExtensionFunction( 
	'glVertex2hNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLhalfNV, constants.GLhalfNV,),
	doc = 'glVertex2hNV( GLhalfNV(x), GLhalfNV(y) ) -> None',
	argNames = ('x', 'y',),
)

glVertex2hvNV = platform.createExtensionFunction( 
	'glVertex2hvNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLushortArray,),
	doc = 'glVertex2hvNV( GLushortArray(v) ) -> None',
	argNames = ('v',),
)

glVertex3hNV = platform.createExtensionFunction( 
	'glVertex3hNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLhalfNV, constants.GLhalfNV, constants.GLhalfNV,),
	doc = 'glVertex3hNV( GLhalfNV(x), GLhalfNV(y), GLhalfNV(z) ) -> None',
	argNames = ('x', 'y', 'z',),
)

glVertex3hvNV = platform.createExtensionFunction( 
	'glVertex3hvNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLushortArray,),
	doc = 'glVertex3hvNV( GLushortArray(v) ) -> None',
	argNames = ('v',),
)

glVertex4hNV = platform.createExtensionFunction( 
	'glVertex4hNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLhalfNV, constants.GLhalfNV, constants.GLhalfNV, constants.GLhalfNV,),
	doc = 'glVertex4hNV( GLhalfNV(x), GLhalfNV(y), GLhalfNV(z), GLhalfNV(w) ) -> None',
	argNames = ('x', 'y', 'z', 'w',),
)

glVertex4hvNV = platform.createExtensionFunction( 
	'glVertex4hvNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLushortArray,),
	doc = 'glVertex4hvNV( GLushortArray(v) ) -> None',
	argNames = ('v',),
)

glNormal3hNV = platform.createExtensionFunction( 
	'glNormal3hNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLhalfNV, constants.GLhalfNV, constants.GLhalfNV,),
	doc = 'glNormal3hNV( GLhalfNV(nx), GLhalfNV(ny), GLhalfNV(nz) ) -> None',
	argNames = ('nx', 'ny', 'nz',),
)

glNormal3hvNV = platform.createExtensionFunction( 
	'glNormal3hvNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLushortArray,),
	doc = 'glNormal3hvNV( GLushortArray(v) ) -> None',
	argNames = ('v',),
)

glColor3hNV = platform.createExtensionFunction( 
	'glColor3hNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLhalfNV, constants.GLhalfNV, constants.GLhalfNV,),
	doc = 'glColor3hNV( GLhalfNV(red), GLhalfNV(green), GLhalfNV(blue) ) -> None',
	argNames = ('red', 'green', 'blue',),
)

glColor3hvNV = platform.createExtensionFunction( 
	'glColor3hvNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLushortArray,),
	doc = 'glColor3hvNV( GLushortArray(v) ) -> None',
	argNames = ('v',),
)

glColor4hNV = platform.createExtensionFunction( 
	'glColor4hNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLhalfNV, constants.GLhalfNV, constants.GLhalfNV, constants.GLhalfNV,),
	doc = 'glColor4hNV( GLhalfNV(red), GLhalfNV(green), GLhalfNV(blue), GLhalfNV(alpha) ) -> None',
	argNames = ('red', 'green', 'blue', 'alpha',),
)

glColor4hvNV = platform.createExtensionFunction( 
	'glColor4hvNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLushortArray,),
	doc = 'glColor4hvNV( GLushortArray(v) ) -> None',
	argNames = ('v',),
)

glTexCoord1hNV = platform.createExtensionFunction( 
	'glTexCoord1hNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLhalfNV,),
	doc = 'glTexCoord1hNV( GLhalfNV(s) ) -> None',
	argNames = ('s',),
)

glTexCoord1hvNV = platform.createExtensionFunction( 
	'glTexCoord1hvNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLushortArray,),
	doc = 'glTexCoord1hvNV( GLushortArray(v) ) -> None',
	argNames = ('v',),
)

glTexCoord2hNV = platform.createExtensionFunction( 
	'glTexCoord2hNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLhalfNV, constants.GLhalfNV,),
	doc = 'glTexCoord2hNV( GLhalfNV(s), GLhalfNV(t) ) -> None',
	argNames = ('s', 't',),
)

glTexCoord2hvNV = platform.createExtensionFunction( 
	'glTexCoord2hvNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLushortArray,),
	doc = 'glTexCoord2hvNV( GLushortArray(v) ) -> None',
	argNames = ('v',),
)

glTexCoord3hNV = platform.createExtensionFunction( 
	'glTexCoord3hNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLhalfNV, constants.GLhalfNV, constants.GLhalfNV,),
	doc = 'glTexCoord3hNV( GLhalfNV(s), GLhalfNV(t), GLhalfNV(r) ) -> None',
	argNames = ('s', 't', 'r',),
)

glTexCoord3hvNV = platform.createExtensionFunction( 
	'glTexCoord3hvNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLushortArray,),
	doc = 'glTexCoord3hvNV( GLushortArray(v) ) -> None',
	argNames = ('v',),
)

glTexCoord4hNV = platform.createExtensionFunction( 
	'glTexCoord4hNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLhalfNV, constants.GLhalfNV, constants.GLhalfNV, constants.GLhalfNV,),
	doc = 'glTexCoord4hNV( GLhalfNV(s), GLhalfNV(t), GLhalfNV(r), GLhalfNV(q) ) -> None',
	argNames = ('s', 't', 'r', 'q',),
)

glTexCoord4hvNV = platform.createExtensionFunction( 
	'glTexCoord4hvNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLushortArray,),
	doc = 'glTexCoord4hvNV( GLushortArray(v) ) -> None',
	argNames = ('v',),
)

glMultiTexCoord1hNV = platform.createExtensionFunction( 
	'glMultiTexCoord1hNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLhalfNV,),
	doc = 'glMultiTexCoord1hNV( GLenum(target), GLhalfNV(s) ) -> None',
	argNames = ('target', 's',),
)

glMultiTexCoord1hvNV = platform.createExtensionFunction( 
	'glMultiTexCoord1hvNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLushortArray,),
	doc = 'glMultiTexCoord1hvNV( GLenum(target), GLushortArray(v) ) -> None',
	argNames = ('target', 'v',),
)

glMultiTexCoord2hNV = platform.createExtensionFunction( 
	'glMultiTexCoord2hNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLhalfNV, constants.GLhalfNV,),
	doc = 'glMultiTexCoord2hNV( GLenum(target), GLhalfNV(s), GLhalfNV(t) ) -> None',
	argNames = ('target', 's', 't',),
)

glMultiTexCoord2hvNV = platform.createExtensionFunction( 
	'glMultiTexCoord2hvNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLushortArray,),
	doc = 'glMultiTexCoord2hvNV( GLenum(target), GLushortArray(v) ) -> None',
	argNames = ('target', 'v',),
)

glMultiTexCoord3hNV = platform.createExtensionFunction( 
	'glMultiTexCoord3hNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLhalfNV, constants.GLhalfNV, constants.GLhalfNV,),
	doc = 'glMultiTexCoord3hNV( GLenum(target), GLhalfNV(s), GLhalfNV(t), GLhalfNV(r) ) -> None',
	argNames = ('target', 's', 't', 'r',),
)

glMultiTexCoord3hvNV = platform.createExtensionFunction( 
	'glMultiTexCoord3hvNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLushortArray,),
	doc = 'glMultiTexCoord3hvNV( GLenum(target), GLushortArray(v) ) -> None',
	argNames = ('target', 'v',),
)

glMultiTexCoord4hNV = platform.createExtensionFunction( 
	'glMultiTexCoord4hNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLhalfNV, constants.GLhalfNV, constants.GLhalfNV, constants.GLhalfNV,),
	doc = 'glMultiTexCoord4hNV( GLenum(target), GLhalfNV(s), GLhalfNV(t), GLhalfNV(r), GLhalfNV(q) ) -> None',
	argNames = ('target', 's', 't', 'r', 'q',),
)

glMultiTexCoord4hvNV = platform.createExtensionFunction( 
	'glMultiTexCoord4hvNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, arrays.GLushortArray,),
	doc = 'glMultiTexCoord4hvNV( GLenum(target), GLushortArray(v) ) -> None',
	argNames = ('target', 'v',),
)

glFogCoordhNV = platform.createExtensionFunction( 
	'glFogCoordhNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLhalfNV,),
	doc = 'glFogCoordhNV( GLhalfNV(fog) ) -> None',
	argNames = ('fog',),
)

glFogCoordhvNV = platform.createExtensionFunction( 
	'glFogCoordhvNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLushortArray,),
	doc = 'glFogCoordhvNV( GLushortArray(fog) ) -> None',
	argNames = ('fog',),
)

glSecondaryColor3hNV = platform.createExtensionFunction( 
	'glSecondaryColor3hNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLhalfNV, constants.GLhalfNV, constants.GLhalfNV,),
	doc = 'glSecondaryColor3hNV( GLhalfNV(red), GLhalfNV(green), GLhalfNV(blue) ) -> None',
	argNames = ('red', 'green', 'blue',),
)

glSecondaryColor3hvNV = platform.createExtensionFunction( 
	'glSecondaryColor3hvNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLushortArray,),
	doc = 'glSecondaryColor3hvNV( GLushortArray(v) ) -> None',
	argNames = ('v',),
)

glVertexWeighthNV = platform.createExtensionFunction( 
	'glVertexWeighthNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLhalfNV,),
	doc = 'glVertexWeighthNV( GLhalfNV(weight) ) -> None',
	argNames = ('weight',),
)

glVertexWeighthvNV = platform.createExtensionFunction( 
	'glVertexWeighthvNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLushortArray,),
	doc = 'glVertexWeighthvNV( GLushortArray(weight) ) -> None',
	argNames = ('weight',),
)

glVertexAttrib1hNV = platform.createExtensionFunction( 
	'glVertexAttrib1hNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLhalfNV,),
	doc = 'glVertexAttrib1hNV( GLuint(index), GLhalfNV(x) ) -> None',
	argNames = ('index', 'x',),
)

glVertexAttrib1hvNV = platform.createExtensionFunction( 
	'glVertexAttrib1hvNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLushortArray,),
	doc = 'glVertexAttrib1hvNV( GLuint(index), GLushortArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttrib2hNV = platform.createExtensionFunction( 
	'glVertexAttrib2hNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLhalfNV, constants.GLhalfNV,),
	doc = 'glVertexAttrib2hNV( GLuint(index), GLhalfNV(x), GLhalfNV(y) ) -> None',
	argNames = ('index', 'x', 'y',),
)

glVertexAttrib2hvNV = platform.createExtensionFunction( 
	'glVertexAttrib2hvNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLushortArray,),
	doc = 'glVertexAttrib2hvNV( GLuint(index), GLushortArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttrib3hNV = platform.createExtensionFunction( 
	'glVertexAttrib3hNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLhalfNV, constants.GLhalfNV, constants.GLhalfNV,),
	doc = 'glVertexAttrib3hNV( GLuint(index), GLhalfNV(x), GLhalfNV(y), GLhalfNV(z) ) -> None',
	argNames = ('index', 'x', 'y', 'z',),
)

glVertexAttrib3hvNV = platform.createExtensionFunction( 
	'glVertexAttrib3hvNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLushortArray,),
	doc = 'glVertexAttrib3hvNV( GLuint(index), GLushortArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttrib4hNV = platform.createExtensionFunction( 
	'glVertexAttrib4hNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLhalfNV, constants.GLhalfNV, constants.GLhalfNV, constants.GLhalfNV,),
	doc = 'glVertexAttrib4hNV( GLuint(index), GLhalfNV(x), GLhalfNV(y), GLhalfNV(z), GLhalfNV(w) ) -> None',
	argNames = ('index', 'x', 'y', 'z', 'w',),
)

glVertexAttrib4hvNV = platform.createExtensionFunction( 
	'glVertexAttrib4hvNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, arrays.GLushortArray,),
	doc = 'glVertexAttrib4hvNV( GLuint(index), GLushortArray(v) ) -> None',
	argNames = ('index', 'v',),
)

glVertexAttribs1hvNV = platform.createExtensionFunction( 
	'glVertexAttribs1hvNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLsizei, arrays.GLushortArray,),
	doc = 'glVertexAttribs1hvNV( GLuint(index), GLsizei(n), GLushortArray(v) ) -> None',
	argNames = ('index', 'n', 'v',),
)

glVertexAttribs2hvNV = platform.createExtensionFunction( 
	'glVertexAttribs2hvNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLsizei, arrays.GLushortArray,),
	doc = 'glVertexAttribs2hvNV( GLuint(index), GLsizei(n), GLushortArray(v) ) -> None',
	argNames = ('index', 'n', 'v',),
)

glVertexAttribs3hvNV = platform.createExtensionFunction( 
	'glVertexAttribs3hvNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLsizei, arrays.GLushortArray,),
	doc = 'glVertexAttribs3hvNV( GLuint(index), GLsizei(n), GLushortArray(v) ) -> None',
	argNames = ('index', 'n', 'v',),
)

glVertexAttribs4hvNV = platform.createExtensionFunction( 
	'glVertexAttribs4hvNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLsizei, arrays.GLushortArray,),
	doc = 'glVertexAttribs4hvNV( GLuint(index), GLsizei(n), GLushortArray(v) ) -> None',
	argNames = ('index', 'n', 'v',),
)


def glInitHalfFloatNV():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



"""Lists/tuples as data-format for storage

Note:
	This implementation is *far* less efficient than using Numpy
	to support lists/tuples, as the code here is all available in
	C-level code there.  This implementation is required to allow
	for usage without numpy installed.
"""
REGISTRY_NAME = 'ctypesarrays'
import ctypes, _ctypes

from OpenGL import constants, constant
from OpenGL.arrays import formathandler
try:
	from OpenGL.arrays import numpymodule
except ImportError, err:
	# we are required...
	HANDLED_TYPES = (list,tuple)
else:
	HANDLED_TYPES = ()
import operator

class ListHandler( formathandler.FormatHandler ):
	"""Storage of array data in Python lists/arrays

	This mechanism, unlike multi-dimensional arrays, is not necessarily
	uniform in type or dimension, so we have to do a lot of extra checks
	to make sure that we get a correctly-structured array.  That, as
	well as the need to copy the arrays in Python code, makes this a far
	less efficient implementation than the numpy implementation, which
	does all the same things, but does them all in C code.

	Note: as an *output* format, this format handler produces ctypes
		arrays, not Python lists, this is done for convenience in coding
		the implementation, mostly.
	"""
	from_param = staticmethod( ctypes.byref )
	dataPointer = staticmethod( ctypes.addressof )
	HANDLED_TYPES = HANDLED_TYPES 
	def voidDataPointer( cls, value ):
		"""Given value in a known data-pointer type, return void_p for pointer"""
		return ctypes.byref( value )
	def zeros( self, dims, typeCode ):
		"""Return array of zeros in given size"""
		type = GL_TYPE_TO_ARRAY_MAPPING[ typeCode ]
		for dim in dims:
			type *= dim 
		return type() # should expicitly set to 0s
	def dimsOf( cls, x ):
		"""Calculate total dimension-set of the elements in x
		
		This is *extremely* messy, as it has to track nested arrays
		where the arrays could be different sizes on all sorts of 
		levels...
		"""
		try:
			dimensions = [ len(x) ]
		except (TypeError,AttributeError,ValueError), err:
			return []
		else:
			childDimension = None
			for child in x:
				newDimension = cls.dimsOf( child )
				if childDimension is not None:
					if newDimension != childDimension:
						raise ValueError( 
							"""Non-uniform array encountered: %s versus %s"""%(
								newDimension, childDimension,
							), x
						)
	dimsOf = classmethod( dimsOf )

	def arrayToGLType( self, value ):
		"""Given a value, guess OpenGL type of the corresponding pointer"""

		result = ARRAY_TO_GL_TYPE_MAPPING.get( value._type_ )
		if result is not None:
			return result
		raise TypeError(
			"""Don't know GL type for array of type %r, known types: %s\nvalue:%s"""%(
				value._type_, ARRAY_TO_GL_TYPE_MAPPING.keys(), value,
			)
		)
	def arraySize( self, value, typeCode = None ):
		"""Given a data-value, calculate dimensions for the array"""
		dims = 1
		for base in self.types( value ):
			length = getattr( base, '_length_', None)
			if length is not None:
				dims *= length
		return dims 
	def types( self, value ):
		"""Produce iterable producing all composite types"""
		dimObject = value
		while dimObject is not None:
			yield dimObject
			dimObject = getattr( dimObject, '_type_', None )
			if isinstance( dimObject, (str,unicode)):
				dimObject = None 
	def dims( self, value ):
		"""Produce iterable of all dimensions"""
		for base in self.types( value ):
			length = getattr( base, '_length_', None)
			if length is not None:
				yield length
	def asArray( self, value, typeCode=None ):
		"""Convert given value to a ctypes array value of given typeCode
		
		This does a *lot* of work just to get the data into the correct
		format.  It's not going to be anywhere near as fast as a numpy
		or similar approach!
		"""
		if typeCode is None:
			raise NotImplementedError( """Haven't implemented type-inference for lists yet""" )
		arrayType = GL_TYPE_TO_ARRAY_MAPPING[ typeCode ]
		if isinstance( value, (list,tuple)):
			subItems = [
				self.asArray( item, typeCode )
				for item in value
			]
			if subItems:
				for dim in self.dimensions( subItems[0] )[::-1]:
					arrayType *= dim
				arrayType *= len( subItems )
				result = arrayType()
				result[:] = subItems
				return result
		else:
			return arrayType( value )
	def unitSize( self, value, typeCode=None ):
		"""Determine unit size of an array (if possible)"""
		return tuple(self.dims(value))[-1]
	def dimensions( self, value, typeCode=None ):
		"""Determine dimensions of the passed array value (if possible)"""
		return tuple( self.dims(value) )


ARRAY_TO_GL_TYPE_MAPPING = {
	constants.GLdouble: constants.GL_DOUBLE,
	constants.GLfloat: constants.GL_FLOAT,
	constants.GLint: constants.GL_INT,
	constants.GLuint: constants.GL_UNSIGNED_INT,
	constants.GLshort: constants.GL_SHORT,
	constants.GLushort: constants.GL_UNSIGNED_SHORT,
		
	constants.GLchar: constants.GL_CHAR,
	constants.GLbyte: constants.GL_BYTE,
	constants.GLubyte: constants.GL_UNSIGNED_BYTE,
}
GL_TYPE_TO_ARRAY_MAPPING = {
	constants.GL_DOUBLE: constants.GLdouble,
	constants.GL_FLOAT: constants.GLfloat,
	constants.GL_INT: constants.GLint,
	constants.GL_UNSIGNED_INT: constants.GLuint,
	constants.GL_SHORT: constants.GLshort,
	constants.GL_UNSIGNED_SHORT: constants.GLushort,
		
	constants.GL_CHAR: constants.GLchar,
	constants.GL_BYTE: constants.GLbyte,
	constants.GL_UNSIGNED_BYTE: constants.GLubyte,
}



'''OpenGL extension OES.read_format

Overview (from the spec)
	
	This extension provides the capability to query an OpenGL
	implementation for a preferred type and format combination
	for use with reading the color buffer with the ReadPixels
	command.  The purpose is to enable embedded implementations
	to support a greatly reduced set of type/format combinations
	and provide a mechanism for applications to determine which
	implementation-specific combination is supported.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/OES/read_format.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_OES_read_format'
GL_IMPLEMENTATION_COLOR_READ_TYPE_OES = constant.Constant( 'GL_IMPLEMENTATION_COLOR_READ_TYPE_OES', 0x8B9A )
GL_IMPLEMENTATION_COLOR_READ_FORMAT_OES = constant.Constant( 'GL_IMPLEMENTATION_COLOR_READ_FORMAT_OES', 0x8B9B )


def glInitReadFormatOES():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.gpu_program_parameters

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/gpu_program_parameters.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_gpu_program_parameters'

glProgramEnvParameters4fvEXT = platform.createExtensionFunction( 
	'glProgramEnvParameters4fvEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint, constants.GLsizei, arrays.GLfloatArray,),
	doc = 'glProgramEnvParameters4fvEXT( GLenum(target), GLuint(index), GLsizei(count), GLfloatArray(params) ) -> None',
	argNames = ('target', 'index', 'count', 'params',),
)

glProgramLocalParameters4fvEXT = platform.createExtensionFunction( 
	'glProgramLocalParameters4fvEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint, constants.GLsizei, arrays.GLfloatArray,),
	doc = 'glProgramLocalParameters4fvEXT( GLenum(target), GLuint(index), GLsizei(count), GLfloatArray(params) ) -> None',
	argNames = ('target', 'index', 'count', 'params',),
)


def glInitGpuProgramParametersEXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



"""Selection-buffer handling code

This code is resonsible for turning gluint *
arrays into structured representations for use
by Python-level code.
"""
def uintToLong( value ):
	if value < 0:
		# array type without a uint, so represented as an int 
		value = (value & 0x7fffffff) + 0x80000000
	return value


class GLSelectRecord( object ):
	"""Minimalist object for storing an OpenGL selection-buffer record
	
	Provides near and far as *float* values by dividing by 
	self.DISTANCE_DIVISOR (2**32-1)
	From the spec:
		Depth values (which are in the range [0,1]) are multiplied by 
		2^32 - 1, before being placed in the hit record.
	
	Names are unmodified, so normally are slices of the array passed in 
	to GLSelectRecord.fromArray( array )
	"""
	DISTANCE_DIVISOR = float((2L**32)-1)
	__slots__ = ('near','far','names')
	def fromArray( cls, array, total ):
		"""Produce list with all records from the array"""
		result = []
		index = 0
		arrayLength = len(array)
		for item in xrange( total ):
			if index + 2 >= arrayLength:
				break
			count = array[index]
			near = array[index+1]
			far = array[index+2]
			names = map(uintToLong, array[index+3:index+3+count])
			result.append(  cls( near, far, names ) )
			index += 3+count
		return result
	fromArray = classmethod( fromArray )
	
	def __init__( self, near, far, names ):
		"""Initialise/store the values"""
		self.near = self.convertDistance( near )
		self.far = self.convertDistance( far )
		self.names = names 
	def convertDistance( self, value ):
		"""Convert a distance value from array uint to 0.0-1.0 range float"""
		return uintToLong( value ) / self.DISTANCE_DIVISOR
	def __getitem__( self, key ):
		"""Allow for treating the record as a three-tuple"""
		if isinstance( key, (int,long)):
			return (self.near,self.far,self.names)[key]
		elif key in self.__slots__:
			try:
				return getattr( self, key )
			except AttributeError, err:
				raise KeyError( """Don't have an index/key %r for %s instant"""%(
					key, self.__class__,
				))



'''OpenGL extension INTEL.parallel_arrays

Overview (from the spec)
	
	This extension adds the ability to format vertex arrays in a way that's 

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/INTEL/parallel_arrays.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_INTEL_parallel_arrays'
GL_PARALLEL_ARRAYS_INTEL = constant.Constant( 'GL_PARALLEL_ARRAYS_INTEL', 0x83F4 )
glget.addGLGetConstant( GL_PARALLEL_ARRAYS_INTEL, (1,) )
GL_VERTEX_ARRAY_PARALLEL_POINTERS_INTEL = constant.Constant( 'GL_VERTEX_ARRAY_PARALLEL_POINTERS_INTEL', 0x83F5 )
GL_NORMAL_ARRAY_PARALLEL_POINTERS_INTEL = constant.Constant( 'GL_NORMAL_ARRAY_PARALLEL_POINTERS_INTEL', 0x83F6 )
GL_COLOR_ARRAY_PARALLEL_POINTERS_INTEL = constant.Constant( 'GL_COLOR_ARRAY_PARALLEL_POINTERS_INTEL', 0x83F7 )
GL_TEXTURE_COORD_ARRAY_PARALLEL_POINTERS_INTEL = constant.Constant( 'GL_TEXTURE_COORD_ARRAY_PARALLEL_POINTERS_INTEL', 0x83F8 )
glVertexPointervINTEL = platform.createExtensionFunction( 
	'glVertexPointervINTEL', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLenum, ctypes.POINTER(ctypes.c_void_p),),
	doc = 'glVertexPointervINTEL( GLint(size), GLenum(type), POINTER(ctypes.c_void_p)(pointer) ) -> None',
	argNames = ('size', 'type', 'pointer',),
)

glNormalPointervINTEL = platform.createExtensionFunction( 
	'glNormalPointervINTEL', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, ctypes.POINTER(ctypes.c_void_p),),
	doc = 'glNormalPointervINTEL( GLenum(type), POINTER(ctypes.c_void_p)(pointer) ) -> None',
	argNames = ('type', 'pointer',),
)

glColorPointervINTEL = platform.createExtensionFunction( 
	'glColorPointervINTEL', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLenum, ctypes.POINTER(ctypes.c_void_p),),
	doc = 'glColorPointervINTEL( GLint(size), GLenum(type), POINTER(ctypes.c_void_p)(pointer) ) -> None',
	argNames = ('size', 'type', 'pointer',),
)

glTexCoordPointervINTEL = platform.createExtensionFunction( 
	'glTexCoordPointervINTEL', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLenum, ctypes.POINTER(ctypes.c_void_p),),
	doc = 'glTexCoordPointervINTEL( GLint(size), GLenum(type), POINTER(ctypes.c_void_p)(pointer) ) -> None',
	argNames = ('size', 'type', 'pointer',),
)


def glInitParallelArraysINTEL():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



"""GLUT functions requiring special handling to provide Pythonic wrappers

Note:
	GLUT callbacks are controlled by a flag in the platform module.  The
	GLUT_GUARD_CALLBACKS flag controls whether to wrap passed functions
	with error-checking and context-validity checking code so that the
	callbacks will only trigger if there is a valid context.  This is done
	so that systems such as Win32 will not continue running GLUT callbacks
	after the system has exited.

	Note:
		This is not a problem with FreeGLUT on Linux, so Linux does not
		add the extra overhead of the wrapper function.
	Note:
		This hack does *not* prevent hanging if there is no GLUT callback
		being triggered.  I.e. if you create a GLUT program that doesn't
		explicitly call exit and doesn't call display or the like in a timer
		then your app will hang on exit on Win32.

XXX the platform-specific stuff should be getting done in the 
platform module *not* in the module here!
"""
from OpenGL.platform import GLUT, CurrentContextIsValid, GLUT_GUARD_CALLBACKS
from OpenGL import contextdata, error, platform, logs
from OpenGL.raw import GLUT as simple
import ctypes, os, sys, traceback
PLATFORM = platform.PLATFORM
FUNCTION_TYPE = simple.CALLBACK_FUNCTION_TYPE

log = logs.getLog( 'OpenGL.GLUT.special' )

if os.name == "nt":
	log.info( """Using NT-specific GLUT calls with exit callbacks""" )
	__glutInitWithExit = platform.createBaseFunction(
		'__glutInitWithExit', dll=platform.GLUT, resultType=None,
		argTypes=[ctypes.POINTER(ctypes.c_int),ctypes.POINTER(ctypes.c_char_p),ctypes.c_void_p],
		doc='glutInit( POINTER(c_int)(pargc), POINTER(STRING)(argv) ) -> None',
		argNames=('pargc', 'argv'),
	)
	__glutCreateWindowWithExit = platform.createBaseFunction(
		'__glutCreateWindowWithExit', dll=platform.GLUT, resultType=ctypes.c_int,
		argTypes=[ctypes.c_char_p, ctypes.c_void_p],
		doc='glutCreateWindow( STRING(title) ) -> c_int',
		argNames=('title',),
	)
	__glutCreateMenuWithExit = platform.createBaseFunction( 
		'__glutCreateMenuWithExit', dll=platform.GLUT, resultType=ctypes.c_int, 
		argTypes=[FUNCTION_TYPE(None, ctypes.c_int), ctypes.c_void_p],
		doc='glutCreateMenu( FUNCTION_TYPE(None, c_int)(callback) ) -> c_int', 
		argNames=('callback',),
	)
	import sys
	_exitfunc = FUNCTION_TYPE( None, ctypes.c_int )(sys.exit)
	
	def _base_glutInit(pargc, argv):
		"""Overrides base glut init with exit-function-aware version"""
		return __glutInitWithExit(pargc, argv, _exitfunc)
	def glutCreateWindow(title):
		"""Create window with given title
		
		This is the Win32-specific version that handles
		registration of an exit-function handler 
		"""
		return __glutCreateWindowWithExit(title, _exitfunc)
	def glutCreateMenu(callback):
		"""Create menu with given callback 
		
		This is the Win32-specific version that handles 
		registration of an exit-function callback.
		"""
		return __glutCreateMenuWithExit(callback, _exitfunc)
else:
	_base_glutInit = getattr(GLUT, 'glutInit', None)
##_base_glutDisplayFunc = GLUT.glutDisplayFunc
##_base_glutIdleFunc = GLUT.glutIdleFunc
##_base_glutEntryFunc = GLUT.glutEntryFunc
##_base_glutReshapeFunc = GLUT.glutReshapeFunc
_base_glutDestroyWindow = getattr(GLUT, 'glutDestroyWindow', None)

class GLUTCallback( object ):
	"""Class implementing GLUT Callback registration functions"""
	def __init__( self, typeName, parameterTypes, parameterNames ):
		"""Initialise the glut callback instance"""
		self.typeName = typeName
		def describe( typ, name ):
			return '(int) %s'%(name)
		self.__doc__ = """Specify handler for GLUT %r events
	def handler( %s ):
		return None"""%( typeName, ", ".join([
			describe( typ,name )
			for (typ,name) in zip( parameterTypes, parameterNames )
		]))
		try:
			self.wrappedOperation = getattr( GLUT, 'glut%sFunc'%(typeName) )
		except AttributeError, err:
			def failFunction( *args, **named ):
				from OpenGL import error
				raise error.NullFunctionError(
					"""Undefined GLUT callback function %s, check for bool(%s) before calling"""%(
						typeName, 'glut%sFunc'%(typeName),
					)
				)
			self.wrappedOperation = failFunction
		self.callbackType = FUNCTION_TYPE( None, *parameterTypes )
		self.CONTEXT_DATA_KEY = 'glut%sFunc'%(typeName, )
	def __call__( self, function, *args ):
		if GLUT_GUARD_CALLBACKS and callable( function ):
			def safeCall( *args, **named ):
				"""Safe calling of GUI callbacks, exits on failures"""
				try:
					if not CurrentContextIsValid():
						raise RuntimeError( """No valid context!""" )
					return function( *args, **named )
				except Exception, err:
					traceback.print_exc()
					sys.stderr.write( """GLUT %s callback %s with %s,%s failed: returning None %s\n"""%(
						self.typeName, function, args, named, err, 
					))
					os._exit(1)
					#return None
			finalFunction = safeCall
		else:
			finalFunction = function
		if callable( finalFunction ):
			cCallback = self.callbackType( finalFunction )
		else:
			cCallback = function
		# keep the function alive as long as the cCallback is...
		#cCallback.function = function
		contextdata.setValue( self.CONTEXT_DATA_KEY, cCallback )
		self.wrappedOperation( cCallback, *args )
		return cCallback
class GLUTTimerCallback( GLUTCallback ):
	"""GLUT timer callbacks (completely nonstandard wrt other GLUT callbacks)"""
	def __call__( self, milliseconds, function, value ):
		cCallback = self.callbackType( function )
		# timers should de-register as soon as they are called...
		# Note: there's no good key to use! we want to allow for
		# multiple instances of the same function with the same value 
		# which means we have nothing that can store it properly...
		callbacks = contextdata.getValue( self.CONTEXT_DATA_KEY )
		if callbacks is None:
			callbacks = []
			contextdata.setValue( self.CONTEXT_DATA_KEY, callbacks )
		def deregister( value ):
			try:
				function( value )
			finally:
				try:
					callbacks.remove( deregister )
				except ValueError, err:
					pass 
				else:
					if not callbacks:
						contextdata.delValue( self.CONTEXT_DATA_KEY )
		cCallback = self.callbackType( deregister )
		#cCallback.function = deregister
		callbacks.append( cCallback )
		self.wrappedOperation( milliseconds, cCallback, value )
		return cCallback

class GLUTMenuCallback( object ):
	"""Place to collect the GLUT Menu manipulation special code"""
	callbackType = FUNCTION_TYPE( ctypes.c_int, ctypes.c_int )
	def glutCreateMenu( cls, func ):
		"""Create a new (current) menu, return small integer identifier
		
		func( int ) -- Function taking a single integer reflecting
			the user's choice, the value passed to glutAddMenuEntry
		
		return menuID (small integer)
		"""
		cCallback = cls.callbackType( func )
		menu = simple.glutCreateMenu( cCallback )
		contextdata.setValue( ('menucallback',menu), (cCallback,func) )
		return menu
	glutCreateMenu.argNames = [ 'func' ]
	glutCreateMenu = classmethod( glutCreateMenu )
	def glutDestroyMenu( cls, menu ):
		"""Destroy (cleanup) the given menu
		
		Deregister's the interal pointer to the menu callback 
		
		returns None
		"""
		result = simple.glutDestroyMenu( menu )
		contextdata.delValue( ('menucallback',menu) )
		return result
	glutDestroyMenu.argNames = [ 'menu' ]
	glutDestroyMenu = classmethod( glutDestroyMenu )

glutCreateMenu = GLUTMenuCallback.glutCreateMenu
#glutCreateMenu.wrappedOperation = simple.glutCreateMenu
glutDestroyMenu = GLUTMenuCallback.glutDestroyMenu
#glutDestroyMenu.wrappedOperation = simple.glutDestroyMenu

glutButtonBoxFunc = GLUTCallback(
	'ButtonBox', (ctypes.c_int,ctypes.c_int), ('button','state'),
)
glutDialsFunc = GLUTCallback(
	'Dials', (ctypes.c_int,ctypes.c_int), ('dial','value'),
)
glutDisplayFunc = GLUTCallback(
	'Display', (), (),
)
glutEntryFunc = GLUTCallback(
	'Entry', (ctypes.c_int,), ('state',),
)
glutIdleFunc = GLUTCallback(
	'Idle', (), (),
)
glutJoystickFunc = GLUTCallback(
	'Joystick', (ctypes.c_uint,ctypes.c_int,ctypes.c_int,ctypes.c_int), ('buttonMask','x','y','z'),
)
glutKeyboardFunc = GLUTCallback(
	'Keyboard', (ctypes.c_char,ctypes.c_int,ctypes.c_int), ('key','x','y'),
)
glutKeyboardUpFunc = GLUTCallback(
	'KeyboardUp', (ctypes.c_char,ctypes.c_int,ctypes.c_int), ('key','x','y'),
)
glutMenuStatusFunc = GLUTCallback(
	'MenuStatus', (ctypes.c_int,ctypes.c_int,ctypes.c_int), ('status','x','y'),
)
glutMenuStateFunc = GLUTCallback(
	'MenuState', (ctypes.c_int,), ('status',),
)
glutMotionFunc = GLUTCallback(
	'Motion', (ctypes.c_int,ctypes.c_int), ('x','y'),
)
glutMouseFunc = GLUTCallback(
	'Mouse', (ctypes.c_int,ctypes.c_int,ctypes.c_int,ctypes.c_int), ('button','state','x','y'),
)
glutOverlayDisplayFunc = GLUTCallback(
	'OverlayDisplay', (), (),
)
glutPassiveMotionFunc = GLUTCallback(
	'PassiveMotion', (ctypes.c_int,ctypes.c_int), ('x','y'),
)
glutReshapeFunc = GLUTCallback(
	'Reshape', (ctypes.c_int,ctypes.c_int), ('width','height'),
)
glutSpaceballButtonFunc = GLUTCallback(
	'SpaceballButton', (ctypes.c_int,ctypes.c_int), ('button','state'),
)
glutSpaceballMotionFunc = GLUTCallback(
	'SpaceballMotion', (ctypes.c_int,ctypes.c_int,ctypes.c_int), ('x','y','z'),
)
glutSpaceballRotateFunc = GLUTCallback(
	'SpaceballRotate', (ctypes.c_int,ctypes.c_int,ctypes.c_int), ('x','y','z'),
)
glutSpecialFunc = GLUTCallback(
	'Special', (ctypes.c_int,ctypes.c_int,ctypes.c_int), ('key','x','y'),
)
glutSpecialUpFunc = GLUTCallback(
	'SpecialUp', (ctypes.c_int,ctypes.c_int,ctypes.c_int), ('key','x','y'),
)
glutTabletButtonFunc = GLUTCallback(
	'TabletButton', (ctypes.c_int,ctypes.c_int,ctypes.c_int,ctypes.c_int), ('button','state','x','y',),
)
glutTabletButtonFunc = GLUTCallback(
	'TabletButton', (ctypes.c_int,ctypes.c_int,ctypes.c_int,ctypes.c_int), ('button','state','x','y',),
)
glutTabletMotionFunc = GLUTCallback(
	'TabletMotion', (ctypes.c_int,ctypes.c_int), ('x','y',),
)
glutVisibilityFunc = GLUTCallback(
	'Visibility', (ctypes.c_int,), ('state',),
)
glutWindowStatusFunc = GLUTCallback(
	'WindowStatus', (ctypes.c_int,), ('state',),
)

# glutTimerFunc is unlike any other GLUT callback-registration...
glutTimerFunc = GLUTTimerCallback(
	'Timer', (ctypes.c_int,), ('value',),
)

INITIALIZED = False
def glutInit( *args ):
	"""Initialise the GLUT library"""
	global INITIALIZED
	if INITIALIZED:
		return args
	INITIALIZED = True
	if args:
		arg,args = args[0],args[1:]
		count = None
		if isinstance(arg, (int,long)):
			# raw API style, (count, values)
			count = arg
			if count != len(args):
				raise ValueError( """Specified count of %s does not match length (%s) of argument list %s"""%(
					count, len(args), args,
				))
		elif isinstance( arg, (str,unicode)):
			# passing in a sequence of strings as individual arguments
			args = (arg,)+args 
			count = len(args)
		else:
			args = arg 
			count = len(args)
	else:
		count=0
		args = []
	args = [str(x) for x in args]
	if not count:
		count, args = 1, ['foo']
	holder = (ctypes.c_char_p * len(args))()
	for i,arg in enumerate(args):
		holder[i] = arg
	count = ctypes.c_int( count )
	import os 
	currentDirectory = os.getcwd()
	try:
		# XXX need to check for error condition here...
		_base_glutInit( ctypes.byref(count), holder )
	finally:
		os.chdir( currentDirectory )
	return [
		str(holder[i]) for i in range( count.value )
	]
glutInit.wrappedOperation = simple.glutInit

def glutDestroyWindow( window ):
	"""Want to destroy the window, we need to do some cleanup..."""
	context = 0
	try:
		GLUT.glutSetWindow(window)
		context = contextdata.getContext()
		result = contextdata.cleanupContext( context )
		log.info( """Cleaning up context data for window %s: %s""", window, result )
	except Exception, err:
		log.error( """Error attempting to clean up context data for GLUT window %s: %s""", window, result )
	return _base_glutDestroyWindow( window )
glutDestroyWindow.wrappedOperation = simple.glutDestroyWindow




'''OpenGL extension ARB.fragment_shader

Overview (from the spec)
	
	This extension adds functionality to define fragment shader objects. A
	fragment shader object is a shader object (see the ARB_shader_objects
	extension) that, when attached to a program object, can be compiled and
	linked to produce an executable that runs on the fragment processor in
	OpenGL. The fragment processor is a programmable unit that replaces the
	OpenGL 1.4 fixed-function texturing, color sum and fog stages. This
	extension also defines how such an executable interacts with the fixed
	functionality fragment processing of OpenGL 1.4. The language used to
	write fragment shaders is not discussed here. That language is defined
	in the OpenGL Shading Language specification as the Fragment Shading
	Language.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/ARB/fragment_shader.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ARB_fragment_shader'
GL_FRAGMENT_SHADER_ARB = constant.Constant( 'GL_FRAGMENT_SHADER_ARB', 0x8B30 )
GL_MAX_FRAGMENT_UNIFORM_COMPONENTS_ARB = constant.Constant( 'GL_MAX_FRAGMENT_UNIFORM_COMPONENTS_ARB', 0x8B49 )
glget.addGLGetConstant( GL_MAX_FRAGMENT_UNIFORM_COMPONENTS_ARB, (1,) )
GL_FRAGMENT_SHADER_DERIVATIVE_HINT_ARB = constant.Constant( 'GL_FRAGMENT_SHADER_DERIVATIVE_HINT_ARB', 0x8B8B )
glget.addGLGetConstant( GL_FRAGMENT_SHADER_DERIVATIVE_HINT_ARB, (1,) )


def glInitFragmentShaderARB():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



"""Array-size annotations for OpenGL.raw.GLU

Automatically generated by the generateraw script, do not edit!
"""
from OpenGL.raw import GLU as raw

from ctypes import *
from OpenGL import platform, arrays
from OpenGL.constant import Constant
from OpenGL import constants as GLconstants
GLvoid = GLconstants.GLvoid



gluCheckExtension = arrays.setInputArraySizeType(
	arrays.setInputArraySizeType(
		raw.gluCheckExtension,
		None, # XXX Could not determine size of argument extName for gluCheckExtension arrays.GLubyteArray
		arrays.GLubyteArray, 
		'extName',
	),
	None, # XXX Could not determine size of argument extString for gluCheckExtension arrays.GLubyteArray
	arrays.GLubyteArray, 
	'extString',
)

gluGetNurbsProperty = arrays.setInputArraySizeType(
	raw.gluGetNurbsProperty,
	None, # XXX Could not determine size of argument data for gluGetNurbsProperty arrays.GLfloatArray
	arrays.GLfloatArray, 
	'data',
)

gluGetTessProperty = arrays.setInputArraySizeType(
	raw.gluGetTessProperty,
	None, # XXX Could not determine size of argument data for gluGetTessProperty arrays.GLdoubleArray
	arrays.GLdoubleArray, 
	'data',
)

gluLoadSamplingMatrices = arrays.setInputArraySizeType(
	arrays.setInputArraySizeType(
		arrays.setInputArraySizeType(
			raw.gluLoadSamplingMatrices,
			None, # XXX Could not determine size of argument model for gluLoadSamplingMatrices arrays.GLfloatArray
			arrays.GLfloatArray, 
			'model',
		),
		None, # XXX Could not determine size of argument perspective for gluLoadSamplingMatrices arrays.GLfloatArray
		arrays.GLfloatArray, 
		'perspective',
	),
	None, # XXX Could not determine size of argument view for gluLoadSamplingMatrices arrays.GLintArray
	arrays.GLintArray, 
	'view',
)

gluNurbsCurve = arrays.setInputArraySizeType(
	arrays.setInputArraySizeType(
		raw.gluNurbsCurve,
		None, # XXX Could not determine size of argument knots for gluNurbsCurve arrays.GLfloatArray
		arrays.GLfloatArray, 
		'knots',
	),
	None, # XXX Could not determine size of argument control for gluNurbsCurve arrays.GLfloatArray
	arrays.GLfloatArray, 
	'control',
)

gluNurbsSurface = arrays.setInputArraySizeType(
	arrays.setInputArraySizeType(
		arrays.setInputArraySizeType(
			raw.gluNurbsSurface,
			None, # XXX Could not determine size of argument sKnots for gluNurbsSurface arrays.GLfloatArray
			arrays.GLfloatArray, 
			'sKnots',
		),
		None, # XXX Could not determine size of argument tKnots for gluNurbsSurface arrays.GLfloatArray
		arrays.GLfloatArray, 
		'tKnots',
	),
	None, # XXX Could not determine size of argument control for gluNurbsSurface arrays.GLfloatArray
	arrays.GLfloatArray, 
	'control',
)

gluPickMatrix = arrays.setInputArraySizeType(
	raw.gluPickMatrix,
	None, # XXX Could not determine size of argument viewport for gluPickMatrix arrays.GLintArray
	arrays.GLintArray, 
	'viewport',
)

gluProject = arrays.setInputArraySizeType(
	arrays.setInputArraySizeType(
		arrays.setInputArraySizeType(
			arrays.setInputArraySizeType(
				arrays.setInputArraySizeType(
					arrays.setInputArraySizeType(
						raw.gluProject,
						None, # XXX Could not determine size of argument model for gluProject arrays.GLdoubleArray
						arrays.GLdoubleArray, 
						'model',
					),
					None, # XXX Could not determine size of argument proj for gluProject arrays.GLdoubleArray
					arrays.GLdoubleArray, 
					'proj',
				),
				None, # XXX Could not determine size of argument view for gluProject arrays.GLintArray
				arrays.GLintArray, 
				'view',
			),
			None, # XXX Could not determine size of argument winX for gluProject arrays.GLdoubleArray
			arrays.GLdoubleArray, 
			'winX',
		),
		None, # XXX Could not determine size of argument winY for gluProject arrays.GLdoubleArray
		arrays.GLdoubleArray, 
		'winY',
	),
	None, # XXX Could not determine size of argument winZ for gluProject arrays.GLdoubleArray
	arrays.GLdoubleArray, 
	'winZ',
)

gluPwlCurve = arrays.setInputArraySizeType(
	raw.gluPwlCurve,
	None, # XXX Could not determine size of argument data for gluPwlCurve arrays.GLfloatArray
	arrays.GLfloatArray, 
	'data',
)

gluTessVertex = arrays.setInputArraySizeType(
	raw.gluTessVertex,
	None, # XXX Could not determine size of argument location for gluTessVertex arrays.GLdoubleArray
	arrays.GLdoubleArray, 
	'location',
)

gluUnProject = arrays.setInputArraySizeType(
	arrays.setInputArraySizeType(
		arrays.setInputArraySizeType(
			arrays.setInputArraySizeType(
				arrays.setInputArraySizeType(
					arrays.setInputArraySizeType(
						raw.gluUnProject,
						None, # XXX Could not determine size of argument model for gluUnProject arrays.GLdoubleArray
						arrays.GLdoubleArray, 
						'model',
					),
					None, # XXX Could not determine size of argument proj for gluUnProject arrays.GLdoubleArray
					arrays.GLdoubleArray, 
					'proj',
				),
				None, # XXX Could not determine size of argument view for gluUnProject arrays.GLintArray
				arrays.GLintArray, 
				'view',
			),
			None, # XXX Could not determine size of argument objX for gluUnProject arrays.GLdoubleArray
			arrays.GLdoubleArray, 
			'objX',
		),
		None, # XXX Could not determine size of argument objY for gluUnProject arrays.GLdoubleArray
		arrays.GLdoubleArray, 
		'objY',
	),
	None, # XXX Could not determine size of argument objZ for gluUnProject arrays.GLdoubleArray
	arrays.GLdoubleArray, 
	'objZ',
)

gluUnProject4 = arrays.setInputArraySizeType(
	arrays.setInputArraySizeType(
		arrays.setInputArraySizeType(
			arrays.setInputArraySizeType(
				arrays.setInputArraySizeType(
					arrays.setInputArraySizeType(
						arrays.setInputArraySizeType(
							raw.gluUnProject4,
							None, # XXX Could not determine size of argument model for gluUnProject4 arrays.GLdoubleArray
							arrays.GLdoubleArray, 
							'model',
						),
						None, # XXX Could not determine size of argument proj for gluUnProject4 arrays.GLdoubleArray
						arrays.GLdoubleArray, 
						'proj',
					),
					None, # XXX Could not determine size of argument view for gluUnProject4 arrays.GLintArray
					arrays.GLintArray, 
					'view',
				),
				None, # XXX Could not determine size of argument objX for gluUnProject4 arrays.GLdoubleArray
				arrays.GLdoubleArray, 
				'objX',
			),
			None, # XXX Could not determine size of argument objY for gluUnProject4 arrays.GLdoubleArray
			arrays.GLdoubleArray, 
			'objY',
		),
		None, # XXX Could not determine size of argument objZ for gluUnProject4 arrays.GLdoubleArray
		arrays.GLdoubleArray, 
		'objZ',
	),
	None, # XXX Could not determine size of argument objW for gluUnProject4 arrays.GLdoubleArray
	arrays.GLdoubleArray, 
	'objW',
)

__all__ = [
	'gluCheckExtension',
	'gluGetNurbsProperty',
	'gluGetTessProperty',
	'gluLoadSamplingMatrices',
	'gluNurbsCurve',
	'gluNurbsSurface',
	'gluPickMatrix',
	'gluProject',
	'gluPwlCurve',
	'gluTessVertex',
	'gluUnProject',
	'gluUnProject4'
]




'''OpenGL extension ARB.map_buffer_range

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/ARB/map_buffer_range.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_ARB_map_buffer_range'
GL_MAP_READ_BIT = constant.Constant( 'GL_MAP_READ_BIT', 0x1 )
GL_MAP_WRITE_BIT = constant.Constant( 'GL_MAP_WRITE_BIT', 0x2 )
GL_MAP_INVALIDATE_RANGE_BIT = constant.Constant( 'GL_MAP_INVALIDATE_RANGE_BIT', 0x4 )
GL_MAP_INVALIDATE_BUFFER_BIT = constant.Constant( 'GL_MAP_INVALIDATE_BUFFER_BIT', 0x8 )
GL_MAP_FLUSH_EXPLICIT_BIT = constant.Constant( 'GL_MAP_FLUSH_EXPLICIT_BIT', 0x10 )
GL_MAP_UNSYNCHRONIZED_BIT = constant.Constant( 'GL_MAP_UNSYNCHRONIZED_BIT', 0x20 )
glMapBufferRange = platform.createExtensionFunction( 
	'glMapBufferRange', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLintptr, constants.GLsizeiptr, constants.GLbitfield,),
	doc = 'glMapBufferRange( GLenum(target), GLintptr(offset), GLsizeiptr(length), GLbitfield(access) ) -> None',
	argNames = ('target', 'offset', 'length', 'access',),
)

glFlushMappedBufferRange = platform.createExtensionFunction( 
	'glFlushMappedBufferRange', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLintptr, constants.GLsizeiptr,),
	doc = 'glFlushMappedBufferRange( GLenum(target), GLintptr(offset), GLsizeiptr(length) ) -> None',
	argNames = ('target', 'offset', 'length',),
)


def glInitMapBufferRangeARB():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension SGIX.blend_alpha_minmax

Overview (from the spec)
	
	Two additional blending equations are specified using the interface
	defined by EXT_blend_minmax.  These equations are similar to the
	MIN_EXT and MAX_EXT blending equations, but the outcome for all four
	color components is determined by a comparison of just the alpha
	component's source and destination values.  These equations are useful
	in image processing and advanced shading algorithms.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/SGIX/blend_alpha_minmax.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_SGIX_blend_alpha_minmax'
GL_ALPHA_MIN_SGIX = constant.Constant( 'GL_ALPHA_MIN_SGIX', 0x8320 )
GL_ALPHA_MAX_SGIX = constant.Constant( 'GL_ALPHA_MAX_SGIX', 0x8321 )


def glInitBlendAlphaMinmaxSGIX():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension VERSION.GL_2_1

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/VERSION/GL_2_1.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_VERSION_GL_2_1'
GL_CURRENT_RASTER_SECONDARY_COLOR = constant.Constant( 'GL_CURRENT_RASTER_SECONDARY_COLOR', 0x845F )
GL_PIXEL_PACK_BUFFER = constant.Constant( 'GL_PIXEL_PACK_BUFFER', 0x88EB )
GL_PIXEL_UNPACK_BUFFER = constant.Constant( 'GL_PIXEL_UNPACK_BUFFER', 0x88EC )
GL_PIXEL_PACK_BUFFER_BINDING = constant.Constant( 'GL_PIXEL_PACK_BUFFER_BINDING', 0x88ED )
GL_PIXEL_UNPACK_BUFFER_BINDING = constant.Constant( 'GL_PIXEL_UNPACK_BUFFER_BINDING', 0x88EF )
GL_FLOAT_MAT2x3 = constant.Constant( 'GL_FLOAT_MAT2x3', 0x8B65 )
GL_FLOAT_MAT2x4 = constant.Constant( 'GL_FLOAT_MAT2x4', 0x8B66 )
GL_FLOAT_MAT3x2 = constant.Constant( 'GL_FLOAT_MAT3x2', 0x8B67 )
GL_FLOAT_MAT3x4 = constant.Constant( 'GL_FLOAT_MAT3x4', 0x8B68 )
GL_FLOAT_MAT4x2 = constant.Constant( 'GL_FLOAT_MAT4x2', 0x8B69 )
GL_FLOAT_MAT4x3 = constant.Constant( 'GL_FLOAT_MAT4x3', 0x8B6A )
GL_SRGB = constant.Constant( 'GL_SRGB', 0x8C40 )
GL_SRGB8 = constant.Constant( 'GL_SRGB8', 0x8C41 )
GL_SRGB_ALPHA = constant.Constant( 'GL_SRGB_ALPHA', 0x8C42 )
GL_SRGB8_ALPHA8 = constant.Constant( 'GL_SRGB8_ALPHA8', 0x8C43 )
GL_SLUMINANCE_ALPHA = constant.Constant( 'GL_SLUMINANCE_ALPHA', 0x8C44 )
GL_SLUMINANCE8_ALPHA8 = constant.Constant( 'GL_SLUMINANCE8_ALPHA8', 0x8C45 )
GL_SLUMINANCE = constant.Constant( 'GL_SLUMINANCE', 0x8C46 )
GL_SLUMINANCE8 = constant.Constant( 'GL_SLUMINANCE8', 0x8C47 )
GL_COMPRESSED_SRGB = constant.Constant( 'GL_COMPRESSED_SRGB', 0x8C48 )
GL_COMPRESSED_SRGB_ALPHA = constant.Constant( 'GL_COMPRESSED_SRGB_ALPHA', 0x8C49 )
GL_COMPRESSED_SLUMINANCE = constant.Constant( 'GL_COMPRESSED_SLUMINANCE', 0x8C4A )
GL_COMPRESSED_SLUMINANCE_ALPHA = constant.Constant( 'GL_COMPRESSED_SLUMINANCE_ALPHA', 0x8C4B )
glUniformMatrix2x3fv = platform.createExtensionFunction( 
	'glUniformMatrix2x3fv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, constants.GLboolean, arrays.GLfloatArray,),
	doc = 'glUniformMatrix2x3fv( GLint(location), GLsizei(count), GLboolean(transpose), GLfloatArray(value) ) -> None',
	argNames = ('location', 'count', 'transpose', 'value',),
)

glUniformMatrix3x2fv = platform.createExtensionFunction( 
	'glUniformMatrix3x2fv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, constants.GLboolean, arrays.GLfloatArray,),
	doc = 'glUniformMatrix3x2fv( GLint(location), GLsizei(count), GLboolean(transpose), GLfloatArray(value) ) -> None',
	argNames = ('location', 'count', 'transpose', 'value',),
)

glUniformMatrix2x4fv = platform.createExtensionFunction( 
	'glUniformMatrix2x4fv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, constants.GLboolean, arrays.GLfloatArray,),
	doc = 'glUniformMatrix2x4fv( GLint(location), GLsizei(count), GLboolean(transpose), GLfloatArray(value) ) -> None',
	argNames = ('location', 'count', 'transpose', 'value',),
)

glUniformMatrix4x2fv = platform.createExtensionFunction( 
	'glUniformMatrix4x2fv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, constants.GLboolean, arrays.GLfloatArray,),
	doc = 'glUniformMatrix4x2fv( GLint(location), GLsizei(count), GLboolean(transpose), GLfloatArray(value) ) -> None',
	argNames = ('location', 'count', 'transpose', 'value',),
)

glUniformMatrix3x4fv = platform.createExtensionFunction( 
	'glUniformMatrix3x4fv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, constants.GLboolean, arrays.GLfloatArray,),
	doc = 'glUniformMatrix3x4fv( GLint(location), GLsizei(count), GLboolean(transpose), GLfloatArray(value) ) -> None',
	argNames = ('location', 'count', 'transpose', 'value',),
)

glUniformMatrix4x3fv = platform.createExtensionFunction( 
	'glUniformMatrix4x3fv', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLsizei, constants.GLboolean, arrays.GLfloatArray,),
	doc = 'glUniformMatrix4x3fv( GLint(location), GLsizei(count), GLboolean(transpose), GLfloatArray(value) ) -> None',
	argNames = ('location', 'count', 'transpose', 'value',),
)




'''OpenGL extension NV.depth_buffer_float

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/NV/depth_buffer_float.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_NV_depth_buffer_float'
GL_DEPTH_COMPONENT32F_NV = constant.Constant( 'GL_DEPTH_COMPONENT32F_NV', 0x8DAB )
GL_DEPTH32F_STENCIL8_NV = constant.Constant( 'GL_DEPTH32F_STENCIL8_NV', 0x8DAC )
GL_FLOAT_32_UNSIGNED_INT_24_8_REV_NV = constant.Constant( 'GL_FLOAT_32_UNSIGNED_INT_24_8_REV_NV', 0x8DAD )
GL_DEPTH_BUFFER_FLOAT_MODE_NV = constant.Constant( 'GL_DEPTH_BUFFER_FLOAT_MODE_NV', 0x8DAF )
glDepthRangedNV = platform.createExtensionFunction( 
	'glDepthRangedNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLdouble, constants.GLdouble,),
	doc = 'glDepthRangedNV( GLdouble(zNear), GLdouble(zFar) ) -> None',
	argNames = ('zNear', 'zFar',),
)

glClearDepthdNV = platform.createExtensionFunction( 
	'glClearDepthdNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLdouble,),
	doc = 'glClearDepthdNV( GLdouble(depth) ) -> None',
	argNames = ('depth',),
)

glDepthBoundsdNV = platform.createExtensionFunction( 
	'glDepthBoundsdNV', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLdouble, constants.GLdouble,),
	doc = 'glDepthBoundsdNV( GLdouble(zmin), GLdouble(zmax) ) -> None',
	argNames = ('zmin', 'zmax',),
)


def glInitDepthBufferFloatNV():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.blend_logic_op

Overview (from the spec)
	
	A single additional blending equation is specified using the interface
	defined by EXT_blend_minmax.  This equation is a simple logical
	combination of the source and destination colors, where the specific
	logical operation is as specified by LogicOp.  While only the XOR
	operation may find wide application, the generality of full logical
	operations is allowed.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/blend_logic_op.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_blend_logic_op'



def glInitBlendLogicOpEXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.secondary_color

Overview (from the spec)
	
	This extension allows specifying the RGB components of the secondary
	color used in the Color Sum stage, instead of using the default
	(0,0,0,0) color. It applies only in RGBA mode and when LIGHTING is
	disabled.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/secondary_color.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_secondary_color'
GL_COLOR_SUM_EXT = constant.Constant( 'GL_COLOR_SUM_EXT', 0x8458 )
glget.addGLGetConstant( GL_COLOR_SUM_EXT, (1,) )
GL_CURRENT_SECONDARY_COLOR_EXT = constant.Constant( 'GL_CURRENT_SECONDARY_COLOR_EXT', 0x8459 )
glget.addGLGetConstant( GL_CURRENT_SECONDARY_COLOR_EXT, (1,) )
GL_SECONDARY_COLOR_ARRAY_SIZE_EXT = constant.Constant( 'GL_SECONDARY_COLOR_ARRAY_SIZE_EXT', 0x845A )
glget.addGLGetConstant( GL_SECONDARY_COLOR_ARRAY_SIZE_EXT, (1,) )
GL_SECONDARY_COLOR_ARRAY_TYPE_EXT = constant.Constant( 'GL_SECONDARY_COLOR_ARRAY_TYPE_EXT', 0x845B )
glget.addGLGetConstant( GL_SECONDARY_COLOR_ARRAY_TYPE_EXT, (1,) )
GL_SECONDARY_COLOR_ARRAY_STRIDE_EXT = constant.Constant( 'GL_SECONDARY_COLOR_ARRAY_STRIDE_EXT', 0x845C )
glget.addGLGetConstant( GL_SECONDARY_COLOR_ARRAY_STRIDE_EXT, (1,) )
GL_SECONDARY_COLOR_ARRAY_POINTER_EXT = constant.Constant( 'GL_SECONDARY_COLOR_ARRAY_POINTER_EXT', 0x845D )
GL_SECONDARY_COLOR_ARRAY_EXT = constant.Constant( 'GL_SECONDARY_COLOR_ARRAY_EXT', 0x845E )
glSecondaryColor3bEXT = platform.createExtensionFunction( 
	'glSecondaryColor3bEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLbyte, constants.GLbyte, constants.GLbyte,),
	doc = 'glSecondaryColor3bEXT( GLbyte(red), GLbyte(green), GLbyte(blue) ) -> None',
	argNames = ('red', 'green', 'blue',),
)

glSecondaryColor3bvEXT = platform.createExtensionFunction( 
	'glSecondaryColor3bvEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLbyteArray,),
	doc = 'glSecondaryColor3bvEXT( GLbyteArray(v) ) -> None',
	argNames = ('v',),
)

glSecondaryColor3dEXT = platform.createExtensionFunction( 
	'glSecondaryColor3dEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLdouble, constants.GLdouble, constants.GLdouble,),
	doc = 'glSecondaryColor3dEXT( GLdouble(red), GLdouble(green), GLdouble(blue) ) -> None',
	argNames = ('red', 'green', 'blue',),
)

glSecondaryColor3dvEXT = platform.createExtensionFunction( 
	'glSecondaryColor3dvEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLdoubleArray,),
	doc = 'glSecondaryColor3dvEXT( GLdoubleArray(v) ) -> None',
	argNames = ('v',),
)

glSecondaryColor3fEXT = platform.createExtensionFunction( 
	'glSecondaryColor3fEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLfloat, constants.GLfloat, constants.GLfloat,),
	doc = 'glSecondaryColor3fEXT( GLfloat(red), GLfloat(green), GLfloat(blue) ) -> None',
	argNames = ('red', 'green', 'blue',),
)

glSecondaryColor3fvEXT = platform.createExtensionFunction( 
	'glSecondaryColor3fvEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLfloatArray,),
	doc = 'glSecondaryColor3fvEXT( GLfloatArray(v) ) -> None',
	argNames = ('v',),
)

glSecondaryColor3iEXT = platform.createExtensionFunction( 
	'glSecondaryColor3iEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLint, constants.GLint,),
	doc = 'glSecondaryColor3iEXT( GLint(red), GLint(green), GLint(blue) ) -> None',
	argNames = ('red', 'green', 'blue',),
)

glSecondaryColor3ivEXT = platform.createExtensionFunction( 
	'glSecondaryColor3ivEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLintArray,),
	doc = 'glSecondaryColor3ivEXT( GLintArray(v) ) -> None',
	argNames = ('v',),
)

glSecondaryColor3sEXT = platform.createExtensionFunction( 
	'glSecondaryColor3sEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLshort, constants.GLshort, constants.GLshort,),
	doc = 'glSecondaryColor3sEXT( GLshort(red), GLshort(green), GLshort(blue) ) -> None',
	argNames = ('red', 'green', 'blue',),
)

glSecondaryColor3svEXT = platform.createExtensionFunction( 
	'glSecondaryColor3svEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLshortArray,),
	doc = 'glSecondaryColor3svEXT( GLshortArray(v) ) -> None',
	argNames = ('v',),
)

glSecondaryColor3ubEXT = platform.createExtensionFunction( 
	'glSecondaryColor3ubEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLubyte, constants.GLubyte, constants.GLubyte,),
	doc = 'glSecondaryColor3ubEXT( GLubyte(red), GLubyte(green), GLubyte(blue) ) -> None',
	argNames = ('red', 'green', 'blue',),
)

glSecondaryColor3ubvEXT = platform.createExtensionFunction( 
	'glSecondaryColor3ubvEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLubyteArray,),
	doc = 'glSecondaryColor3ubvEXT( GLubyteArray(v) ) -> None',
	argNames = ('v',),
)

glSecondaryColor3uiEXT = platform.createExtensionFunction( 
	'glSecondaryColor3uiEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLuint, constants.GLuint,),
	doc = 'glSecondaryColor3uiEXT( GLuint(red), GLuint(green), GLuint(blue) ) -> None',
	argNames = ('red', 'green', 'blue',),
)

glSecondaryColor3uivEXT = platform.createExtensionFunction( 
	'glSecondaryColor3uivEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLuintArray,),
	doc = 'glSecondaryColor3uivEXT( GLuintArray(v) ) -> None',
	argNames = ('v',),
)

glSecondaryColor3usEXT = platform.createExtensionFunction( 
	'glSecondaryColor3usEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLushort, constants.GLushort, constants.GLushort,),
	doc = 'glSecondaryColor3usEXT( GLushort(red), GLushort(green), GLushort(blue) ) -> None',
	argNames = ('red', 'green', 'blue',),
)

glSecondaryColor3usvEXT = platform.createExtensionFunction( 
	'glSecondaryColor3usvEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLushortArray,),
	doc = 'glSecondaryColor3usvEXT( GLushortArray(v) ) -> None',
	argNames = ('v',),
)

glSecondaryColorPointerEXT = platform.createExtensionFunction( 
	'glSecondaryColorPointerEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLint, constants.GLenum, constants.GLsizei, ctypes.c_void_p,),
	doc = 'glSecondaryColorPointerEXT( GLint(size), GLenum(type), GLsizei(stride), c_void_p(pointer) ) -> None',
	argNames = ('size', 'type', 'stride', 'pointer',),
)


def glInitSecondaryColorEXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension SGIS.fog_function

Overview (from the spec)
	
	This extension allows to define application-specific fog blend-factor
	function.  Function is defined by the set of the "control" points and
	should be monotonic. Each control point represented as a pair of the
	eye-space distance value and corresponding value of the fog blending 
	factor. The minimum number of control points is one. The maximum 
	number is implementation dependent.
	

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/SGIS/fog_function.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_SGIS_fog_function'
GL_FOG_FUNC_SGIS = constant.Constant( 'GL_FOG_FUNC_SGIS', 0x812A )
GL_FOG_FUNC_POINTS_SGIS = constant.Constant( 'GL_FOG_FUNC_POINTS_SGIS', 0x812B )
GL_MAX_FOG_FUNC_POINTS_SGIS = constant.Constant( 'GL_MAX_FOG_FUNC_POINTS_SGIS', 0x812C )
glFogFuncSGIS = platform.createExtensionFunction( 
	'glFogFuncSGIS', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLsizei, arrays.GLfloatArray,),
	doc = 'glFogFuncSGIS( GLsizei(n), GLfloatArray(points) ) -> None',
	argNames = ('n', 'points',),
)

glGetFogFuncSGIS = platform.createExtensionFunction( 
	'glGetFogFuncSGIS', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(arrays.GLfloatArray,),
	doc = 'glGetFogFuncSGIS( GLfloatArray(points) ) -> None',
	argNames = ('points',),
)


def glInitFogFunctionSGIS():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension EXT.blend_minmax

Overview (from the spec)
	
	Blending capability is extended by respecifying the entire blend
	equation.  While this document defines only two new equations, the
	BlendEquationEXT procedure that it defines will be used by subsequent
	extensions to define additional blending equations.
	
	The two new equations defined by this extension produce the minimum
	(or maximum) color components of the source and destination colors.
	Taking the maximum is useful for applications such as maximum projection
	in medical imaging.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/blend_minmax.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_blend_minmax'
GL_FUNC_ADD_EXT = constant.Constant( 'GL_FUNC_ADD_EXT', 0x8006 )
GL_MIN_EXT = constant.Constant( 'GL_MIN_EXT', 0x8007 )
GL_MAX_EXT = constant.Constant( 'GL_MAX_EXT', 0x8008 )
GL_BLEND_EQUATION_EXT = constant.Constant( 'GL_BLEND_EQUATION_EXT', 0x8009 )
glget.addGLGetConstant( GL_BLEND_EQUATION_EXT, (1,) )
glBlendEquationEXT = platform.createExtensionFunction( 
	'glBlendEquationEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum,),
	doc = 'glBlendEquationEXT( GLenum(mode) ) -> None',
	argNames = ('mode',),
)


def glInitBlendMinmaxEXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



"""VertexBufferObject helper class"""
from OpenGL import GL
from OpenGL.arrays.arraydatatype import ArrayDatatype
from OpenGL.arrays.formathandler import FormatHandler
from OpenGL.GL.ARB import vertex_buffer_object


import weakref
__all__ = ('VBO','VBOHandler')

class Implementation( object ):
	"""Abstraction point for the various implementations that can be used
	"""
	available = False
	def _arbname( self, name ):
		return (
			(name.startswith( 'gl' ) and name.endswith( 'ARB' )) or 
			(name.startswith( 'GL_' ) and name.endswith( 'ARB' ))
		) and (name != 'glInitVertexBufferObjectARB')
	def basename( self, name ):
		if name.endswith( '_ARB' ):
			return name[:-4]
		elif name.endswith( 'ARB' ):
			return name[:-3]
		else:
			return name
	def __init__( self ):
		names = [name for name in dir(vertex_buffer_object) if self._arbname( name )]
		if not GL.glBufferData:
			for name in names:
				setattr( self, self.basename(name), getattr( vertex_buffer_object, name ))
			self.available = True
		elif vertex_buffer_object.glBufferDataARB:
			for name in names:
				setattr( self, self.basename(name), getattr( GL, self.basename(name) ))
			self.available = True
	def __nonzero__( self ):
		return self.available

def get_implementation( *args ):
	"""Retrieve the appropriate implementation for this machine"""
	if VBO._I_ is None:
		VBO.implementation = VBO._I_ = Implementation()
	return VBO._I_


class VBO( object ):
	"""Instances can be passed into array-handling routines
	
	You can check for whether VBOs are supported by accessing the implementation
	attribute of the VBO, which will raise a RuntimeError if there is no available 
	implementation.
	"""
	_DELETERS_ = {}
	copied = False
	_no_cache_ = True # do not cache in context data arrays
	def __init__( 
		self, data, usage='GL_DYNAMIC_DRAW', 
		target='GL_ARRAY_BUFFER',
	):
		self.data = data 
		self.usage = usage 
		self.target = target 
		self.buffers = []
		self._copy_segments = []
	_I_ = None
	implementation = property( get_implementation, )
	def resolve( self, value ):
		"""Resolve string constant to constant"""
		if isinstance( value, (str,unicode)):
			return getattr( self.implementation, self.implementation.basename( value ) )
		return value
	def set_array( self, data ):
		"""Update our entire array with new data"""
		self.data = data 
		self.copied = False
	def __setitem__( self, slice, array):
		"""Set slice of data on the array and vbo (if copied already)
		
		slice -- the Python slice object determining how the data should 
			be copied into the vbo/array 
		array -- something array-compatible that will be used as the 
			source of the data, note that the data-format will have to 
			be the same as the internal data-array to work properly, if 
			not, the amount of data copied will be wrong.
		
		This is a reasonably complex operation, it has to have all sorts
		of state-aware changes to correctly map the source into the low-level
		OpenGL view of the buffer (which is just bytes as far as the GL 
		is concerned).
		"""
		if slice.step and not slice.step == 1:
			raise NotImplemented( """Don't know how to map stepped arrays yet""" )
		# TODO: handle e.g. mapping character data into an integer data-set
		data = ArrayDatatype.asArray( array )
		start = (slice.start or 0) 
		stop = (slice.stop or len(self.data))
		if start < 0:
			start += len(self.data)
			start = max((start,0))
		if stop < 0:
			stop += len(self.data)
			stop = max((stop,0))
		self.data[ slice ] = data
		if self.copied and self.buffers:
			if start-stop != len(data):
				self.copied = False
			elif start-stop == len(self.data):
				# re-copy the whole data-set
				self.copied = False
			elif len(data):
				# now the fun part, we need to make the array match the 
				# structure of the array we're going to copy into and make 
				# the "size" parameter match the value we're going to copy in,
				# note that a 2D array (rather than a 1D array) may require 
				# multiple mappings to copy into the memory area...
				
				# find the step size from the dimensions and base size...
				size = ArrayDatatype.arrayByteCount( data ) / len(array)
				#baseSize = ArrayDatatype.unitSize( data )
				# now create the start and distance values...
				start *= size
				stop *= size
				# wait until the last moment (bind) to copy the data...
				self._copy_segments.append(
					(start,(stop-start), data)
				)
	def __len__( self ):
		return len( self.data )
	def __getattr__( self, key ):
		if key not in ('data','usage','target','buffers', 'copied','_I_','implementation','_copy_segments' ):
			return getattr( self.data, key )
		else:
			raise AttributeError( key )
	def create_buffers( self ):
		"""Create the internal buffer(s)"""
		assert not self.buffers, """Already created the buffer"""
		self.buffers = [ long(self.implementation.glGenBuffers(1)) ]
		self._DELETERS_[ id(self) ] = weakref.ref( self, deleter( self.buffers, id(self), self.implementation ))
		return self.buffers
	def copy_data( self ):
		"""Copy our data into the buffer on the GL side"""
		assert self.buffers, """Should do create_buffers before copy_data"""
		if self.copied:
			if self._copy_segments:
				while self._copy_segments:
					start,size,data  = self._copy_segments.pop(0)
					dataptr = ArrayDatatype.voidDataPointer( data )
					self.implementation.glBufferSubData(self.resolve(self.target), start, size, dataptr)
		else:
			self.implementation.glBufferData(
				self.resolve(self.target), 
				self.data,
				self.resolve(self.usage)
			)
			self.copied = True
	def delete( self ):
		"""Delete this buffer explicitly"""
		if self.buffers:
			while self.buffers:
				try:
					self.implementation.glDeleteBuffers(1, self.buffers.pop(0))
				except AttributeError, err:
					pass
	def bind( self ):
		"""Bind this buffer for use in vertex calls"""
		if not self.buffers:
			buffers = self.create_buffers()
		self.implementation.glBindBuffer( self.resolve(self.target), self.buffers[0])
		self.copy_data()
	def unbind( self ):
		"""Unbind the buffer (make normal array operations active)"""
		self.implementation.glBindBuffer( self.resolve(self.target),0 )

def deleter( buffers, key, implementation ):
	"""Produce a deleter callback to delete the given buffer"""
	def doBufferDeletion( *args, **named ):
		for buffer in buffers:
			try:
				implementation.glDeleteBuffers(1, buffer)
			except AttributeError, err:
				pass
		try:
			VBO._DELETERS_.pop( key )
		except KeyError, err:
			pass
	return doBufferDeletion

class VBOHandler( FormatHandler ):
	"""Handles VBO instances passed in as array data"""
	def dataPointer( self, instance ):
		"""Retrieve data-pointer from the instance's data
		
		Is always NULL, to indicate use of the bound pointer
		"""
		return 0
	def from_param( self, instance ):
		return ctypes.c_void_p( 0 )
	def zeros( self, dims, typeCode ):
		"""Not implemented"""
		raise NotImplemented( """Don't have VBO output support yet""" )
	ones = zeros 
	def asArray( self, value, typeCode=None ):
		"""Given a value, convert to array representation"""
		return value
	def arrayToGLType( self, value ):
		"""Given a value, guess OpenGL type of the corresponding pointer"""
		return ArrayDatatype.arrayToGLType( value.data )
	def arraySize( self, value, typeCode = None ):
		"""Given a data-value, calculate dimensions for the array"""
		return ArrayDatatype.arraySize( value.data )
	def unitSize( self, value, typeCode=None ):
		"""Determine unit size of an array (if possible)"""
		return ArrayDatatype.unitSize( value.data )
	def dimensions( self, value, typeCode=None ):
		"""Determine dimensions of the passed array value (if possible)"""
		return ArrayDatatype.dimensions( value.data )

HANDLER = VBOHandler()
HANDLER.loadAll() # otherwise just the VBO would get loaded :)
HANDLER.register( [ VBO ] )



'''OpenGL extension VERSION.GL_1_2

This module customises the behaviour of the 
OpenGL.raw.GL.VERSION.GL_1_2 to provide a more 
Python-friendly API
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions, wrapper
from OpenGL.GL import glget
import ctypes
from OpenGL.raw.GL.VERSION.GL_1_2 import *
### END AUTOGENERATED SECTION
from OpenGL.GL.VERSION.GL_1_2_images import *

glDrawRangeElements = wrapper.wrapper( simple.glDrawRangeElements ).setPyConverter(
	'indices', arrays.AsArrayOfType( 'indices', 'type' ),
).setCResolver(
	'indices', arrays.ArrayDatatype.voidDataPointer ,
).setReturnValues(
	wrapper.returnPyArgument( 'indices' )
)


'''OpenGL extension EXT.draw_buffers2

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/EXT/draw_buffers2.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_EXT_draw_buffers2'

glColorMaskIndexedEXT = platform.createExtensionFunction( 
	'glColorMaskIndexedEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLuint, constants.GLboolean, constants.GLboolean, constants.GLboolean, constants.GLboolean,),
	doc = 'glColorMaskIndexedEXT( GLuint(index), GLboolean(r), GLboolean(g), GLboolean(b), GLboolean(a) ) -> None',
	argNames = ('index', 'r', 'g', 'b', 'a',),
)

glGetBooleanIndexedvEXT = platform.createExtensionFunction( 
	'glGetBooleanIndexedvEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint, ctypes.POINTER(constants.GLboolean),),
	doc = 'glGetBooleanIndexedvEXT( GLenum(target), GLuint(index), POINTER(constants.GLboolean)(data) ) -> None',
	argNames = ('target', 'index', 'data',),
)

glGetIntegerIndexedvEXT = platform.createExtensionFunction( 
	'glGetIntegerIndexedvEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint, arrays.GLintArray,),
	doc = 'glGetIntegerIndexedvEXT( GLenum(target), GLuint(index), GLintArray(data) ) -> None',
	argNames = ('target', 'index', 'data',),
)

glEnableIndexedEXT = platform.createExtensionFunction( 
	'glEnableIndexedEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint,),
	doc = 'glEnableIndexedEXT( GLenum(target), GLuint(index) ) -> None',
	argNames = ('target', 'index',),
)

glDisableIndexedEXT = platform.createExtensionFunction( 
	'glDisableIndexedEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(constants.GLenum, constants.GLuint,),
	doc = 'glDisableIndexedEXT( GLenum(target), GLuint(index) ) -> None',
	argNames = ('target', 'index',),
)

glIsEnabledIndexedEXT = platform.createExtensionFunction( 
	'glIsEnabledIndexedEXT', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=constants.GLboolean, 
	argTypes=(constants.GLenum, constants.GLuint,),
	doc = 'glIsEnabledIndexedEXT( GLenum(target), GLuint(index) ) -> constants.GLboolean',
	argNames = ('target', 'index',),
)


def glInitDrawBuffers2EXT():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



"""Base class for platform implementations
"""
import ctypes
from OpenGL.platform import ctypesloader
import sys
import OpenGL as top_level_module
from OpenGL import logs

class BasePlatform( object ):
	"""Base class for per-platform implementations
	
	Attributes of note:
	
		EXPORTED_NAMES -- set of names exported via the platform 
			module's namespace...
	
		GL, GLU, GLUT, GLE, OpenGL -- ctypes libraries
	
		DEFAULT_FUNCTION_TYPE -- used as the default function 
			type for functions unless overridden on a per-DLL
			basis with a "FunctionType" member
		
		GLUT_GUARD_CALLBACKS -- if True, the GLUT wrappers 
			will provide guarding wrappers to prevent GLUT 
			errors with uninitialised GLUT.
		
		EXTENSIONS_USE_BASE_FUNCTIONS -- if True, uses regular
			dll attribute-based lookup to retrieve extension 
			function pointers.
	"""
	
	EXPORTED_NAMES = [
		'GetCurrentContext','CurrentContextIsValid','safeGetError',
		'createBaseFunction', 'createExtensionFunction', 'copyBaseFunction',
		'GL','GLU','GLUT','GLE','OpenGL',
		'getGLUTFontPointer',
		'GLUT_GUARD_CALLBACKS',
	]

	
	DEFAULT_FUNCTION_TYPE = None
	GLUT_GUARD_CALLBACKS = False
	EXTENSIONS_USE_BASE_FUNCTIONS = False
	
	def install( self, namespace ):
		"""Install this platform instance into the platform module"""
		for name in self.EXPORTED_NAMES:
			namespace[ name ] = getattr(self,name)
		namespace['PLATFORM'] = self
		return self
	
	def functionTypeFor( self, dll ):
		"""Given a DLL, determine appropriate function type..."""
		if hasattr( dll, 'FunctionType' ):
			return dll.FunctionType
		else:
			return self.DEFAULT_FUNCTION_TYPE
	
	def errorChecking( self, func ):
		"""Add error checking to the function if appropriate"""
		from OpenGL import error
		if top_level_module.ERROR_CHECKING:
			func.errcheck = error.glCheckError
		return func
	def wrapLogging( self, func ):
		"""Wrap function with logging operations if appropriate"""
		return logs.logOnFail( func, logs.getLog( 'OpenGL.errors' ))
	
	def constructFunction( 
		self,
		functionName, dll, 
		resultType=ctypes.c_int, argTypes=(),
		doc = None, argNames = (),
		extension = None,
	):
		"""Core operation to create a new base ctypes function
		
		raises AttributeError if can't find the procedure...
		"""
		if extension and not self.checkExtension( extension ):
			raise AttributeError( """Extension not available""" )
		if extension and not self.EXTENSIONS_USE_BASE_FUNCTIONS:
			# what about the VERSION values???
			if self.checkExtension( extension ):
				pointer = self.getExtensionProcedure( functionName )
				if pointer:
					func = self.functionTypeFor( dll )(
						resultType,
						*argTypes
					)(
						pointer
					)
				else:
					AttributeError( """Extension %r available, but no pointer for function %r"""%(extension,functionName))
			else:
				raise AttributeError( """No extension %r"""%(extension,))
		else:
			func = ctypesloader.buildFunction(
				self.functionTypeFor( dll )(
					resultType,
					*argTypes
				),
				functionName,
				dll,
			)
		func.__doc__ = doc 
		func.argNames = list(argNames or ())
		func.__name__ = functionName
		func.DLL = dll
		func.extension = extension
		func = self.wrapLogging( self.errorChecking( func ))
		return func

	def createBaseFunction( 
		self,
		functionName, dll, 
		resultType=ctypes.c_int, argTypes=(),
		doc = None, argNames = (),
		extension = None,
	):
		"""Create a base function for given name
		
		Normally you can just use the dll.name hook to get the object,
		but we want to be able to create different bindings for the 
		same function, so we do the work manually here to produce a
		base function from a DLL.
		"""
		from OpenGL import wrapper
		try:
			return self.constructFunction(
				functionName, dll, 
				resultType=resultType, argTypes=argTypes,
				doc = doc, argNames = argNames,
				extension = extension,
			)
		except AttributeError, err:
			return self.nullFunction( 
				functionName, dll=dll,
				resultType=resultType, 
				argTypes=argTypes,
				doc = doc, argNames = argNames,
				extension = extension,
			)
	def checkExtension( self, name ):
		"""Check whether the given extension is supported by current context"""
		if not name:
			return True
		context = self.GetCurrentContext()
		if context:
			from OpenGL import contextdata
			from OpenGL.raw.GL import GL_EXTENSIONS
			set = contextdata.getValue( GL_EXTENSIONS, context=context )
			if set is None:
				set = {}
				contextdata.setValue( 
					GL_EXTENSIONS, set, context=context, weak=False 
				)
			current = set.get( name )
			if current is None:
				from OpenGL import extensions
				result = extensions.hasGLExtension( name )
				set[name] = result 
				return result
			return current
		else:
			return False
	createExtensionFunction = createBaseFunction

	def copyBaseFunction( self, original ):
		"""Create a new base function based on an already-created function
		
		This is normally used to provide type-specific convenience versions of
		a definition created by the automated generator.
		"""
		from OpenGL import wrapper, error
		if isinstance( original, _NullFunctionPointer ):
			return self.nullFunction(
				original.__name__,
				original.DLL,
				resultType = original.restype,
				argTypes= original.argtypes,
				doc = original.__doc__,
				argNames = original.argNames,
				extension = original.extension,
			)
		elif hasattr( original, 'originalFunction' ):
			original = original.originalFunction
		return self.createBaseFunction(
			original.__name__, original.DLL, 
			resultType=original.restype, argTypes=original.argtypes,
			doc = original.__doc__, argNames = original.argNames,
			extension = original.argNames,
		)
	def nullFunction( 
		self,
		functionName, dll,
		resultType=ctypes.c_int, 
		argTypes=(),
		doc = None, argNames = (),
		extension = None,
	):
		"""Construct a "null" function pointer"""
		cls = type( functionName, (_NullFunctionPointer,), {
			'__doc__': doc,
		} )
		return cls(
			functionName, dll, resultType, argTypes, argNames, extension=extension, doc=doc,
		)
	def GetCurrentContext( self ):
		"""Retrieve opaque pointer for the current context"""
		raise NotImplementedError( 
			"""Platform does not define a GetCurrentContext function""" 
		)
	def CurrentContextIsValid( self ):
		"""Return boolean of whether current context is valid"""
		raise NotImplementedError( 
			"""Platform does not define a CurrentContextIsValid function""" 
		)
	def getGLUTFontPointer(self, constant ):
		"""Retrieve a GLUT font pointer for this platform"""
		raise NotImplementedError( 
			"""Platform does not define a GLUT font retrieval function""" 
		)
	def safeGetError( self ):
		"""Safety-checked version of glError() call (checks for valid context first)"""
		raise NotImplementedError( 
			"""Platform does not define a safeGetError function""" 
		)

class _NullFunctionPointer( object ):
	"""Function-pointer-like object for undefined functions"""
	def __init__( self, name, dll, resultType, argTypes, argNames, extension=None, doc=None ):
		from OpenGL import error
		self.__name__ = name
		self.DLL = dll
		self.argNames = argNames
		self.argtypes = argTypes
		if top_level_module.ERROR_CHECKING:
			self.errcheck = error.glCheckError
		else:
			self.errcheck = None
		self.restype = resultType
		self.extension = extension
		self.doc = doc
	resolved = False
	def __nonzero__( self ):
		"""Make this object appear to be NULL"""
		if self.extension and not self.resolved:
			self.load()
		return self.resolved
	def load( self ):
		"""Attempt to load the function again, presumably with a context this time"""
		from OpenGL import platform
		if not platform.PLATFORM.checkExtension( self.extension ):
			return None
		try:
			func = platform.PLATFORM.constructFunction(
				self.__name__, self.DLL, 
				resultType=self.restype, 
				argTypes=self.argtypes,
				doc = self.doc, 
				argNames = self.argNames,
				extension = self.extension,
			)
		except AttributeError, err:
			return None 
		else:
			# now short-circuit so that we don't need to check again...
			self.__class__.__call__ = staticmethod( func.__call__ )
			self.resolved = True
			return func
		return None
	def __call__( self, *args, **named ):
		if self.load():
			return self( *args, **named )
		else:
			from OpenGL import error
			raise error.NullFunctionError(
				"""Attempt to call an undefined function %s, check for bool(%s) before calling"""%(
					self.__name__, self.__name__,
				)
			)



'''OpenGL extension GREMEDY.frame_terminator

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/GREMEDY/frame_terminator.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_GREMEDY_frame_terminator'

glFrameTerminatorGREMEDY = platform.createExtensionFunction( 
	'glFrameTerminatorGREMEDY', dll=platform.GL,
	extension=EXTENSION_NAME,
	resultType=None, 
	argTypes=(),
	doc = 'glFrameTerminatorGREMEDY(  ) -> None',
	argNames = (),
)


def glInitFrameTerminatorGREMEDY():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )



'''OpenGL extension NV.vertex_program2_option

Overview (from the spec)
	
	This extension provides additional vertex program functionality
	to extend the standard ARB_vertex_program language and execution
	environment.  ARB programs wishing to use this added functionality
	need only add:
	
	    OPTION NV_vertex_program2;
	
	to the beginning of their vertex programs.
	
	The functionality provided by this extension, which is roughly
	equivalent to that provided by the NV_vertex_program2 extension,
	includes:
	
	  * general purpose dynamic branching,
	
	  * subroutine calls,
	
	  * data-dependent conditional write masks,
	
	  * programmable user clip distances,
	
	  * address registers with four components (instead of just one),
	
	  * absolute value operator on scalar and swizzled operand loads,
	
	  * rudimentary address register math,
	
	  * SIN and COS trigonometry instructions, and
	
	  * fully orthogonal "set on" instructions, including a "set sign"
	    instruction.

The official definition of this extension is available here:
	http://oss.sgi.com/projects/ogl-sample/registry/NV/vertex_program2_option.txt

Automatically generated by the get_gl_extensions script, do not edit!
'''
from OpenGL import platform, constants, constant, arrays
from OpenGL import extensions
from OpenGL.GL import glget
import ctypes
EXTENSION_NAME = 'GL_NV_vertex_program2_option'



def glInitVertexProgram2OptionNV():
	'''Return boolean indicating whether this extension is available'''
	return extensions.hasGLExtension( EXTENSION_NAME )
